#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from __future__ import division
import sys,os
import ConfigParser
import subprocess as sp

# Parameters
# rootdir =  root folder
# templatedir =  template folder
# tmpdir = folder used by lh_build for generating image
# imgdir = image folder (all produced image will be there)
# hookdir =  hook scripts folder
# pkgdir = folder for your local/personal packages


def dircheck(config):
    """
        Check the existence of main directories and configfile.
        If a directory does not exist, just creates it with a default path.
        If the config file does not exist, just creates it in /etc/livecreator.conf.
    """
    WriteNewOptionConfig(config, "intro")
    WriteNewOptionConfig(config, "version")
    try:
        rootdir = config.get("main","rootdir")
        templatedir = config.get("main","templatedir")
        tmpdir = config.get("main","tmpdir")
        imgdir = config.get("main","imgdir")
        hookdir = config.get("main","hookdir")
        pkgdir = config.get("main","pkgdir")
        version = config.get("main","version")
        depurl = config.get("main","depurl")
        secudepurl = config.get("main","secudepurl")
        ftpproxy = config.get("main","ftpproxy")
        httpproxy = config.get("main","httpproxy")
        if not os.path.isdir(rootdir):
            os.makedirs(rootdir,0755)
        if not os.path.isdir(templatedir):
            os.makedirs(templatedir,0755)
        if not os.path.isdir(tmpdir):
            os.makedirs(tmpdir,0700)
        if not os.path.isdir(imgdir):
            os.makedirs(imgdir,0755)
        if not os.path.isdir(hookdir):
            os.makedirs(hookdir,0755)
        if not os.path.isdir(pkgdir):
            os.makedirs(pkgdir,0755)
    except ConfigParser.NoSectionError, s:
        WriteSectionconfig(config, s.section)
        dircheck(config)
    except ConfigParser.NoOptionError,o:
        WriteNewOptionConfig(config, o.option)
        dircheck(config)
    except Exception, e:
        writeNewConfig()


def allPreferences():
    """
       This function return all preferences and
       their default value
    """
    hoption= {'rootdir':'/var/lib/livecreator/',
            'templatedir':'/var/lib/livecreator/templates/',
            'tmpdir':'/var/lib/livecreator/tmp/',
            'tmpdir':'/var/lib/livecreator/tmp/',
            'imgdir':'/var/lib/livecreator/img/',
            'hookdir':'/var/lib/livecreator/hook/',
            'pkgdir':'/var/lib/livecreator/packages/',
            'depurl':'http://ftp.debian.org/debian',
            'secudepurl':'http://security.debian.org',
            'ftpproxy':'',
            'httpproxy':'',
#            'intro' : '''LiveCreator est un outil graphique permettant de créer des Lives CD ou Lives USB en quelques clics.\nSon utilisation intuitive vous permettra de créer une collection de modèles aisément et de les modifier simplement.\n\nPour créer rapidement un nouveau Live, cliquez sur le bouton "suivant" et créez un nouveau modele.\nChoisissez les paquets que vous souhaitez inclure puis les options dans les deux parties suivantes.\nEnfin, générez l'image à l'aide de l'étape "Générer l'image"\n\nVous pouvez utiliser le panneau de droite pour passer d'une partie à l'autre.\n\n\n\n\n\t\t\t\t\t\t\t\t\t''',
            'intro' : _('''What's LiveCreator by the way ?\n\n\nIt's an easy to use graphical interface to build images. Images can be for CD (iso) or for USB keys. To start and build an image in a simple an rapid way, just click "Next" button an create a new template.\nThen you just have to choose the paquets that you want include in your image by some clicks and enventually some options.\nFinnaly, you can launch process clicking "Generate image"... That's all !\n\n\nIt's possible to use the left panel to pass rapidly from a section to another.\n\n\n\n\n\t\t\t\t\t\t\t\t\t'''),
            'version':"0.1"}
    return hoption


def WriteSectionconfig(config,section):
    """
        Write the main section in the config file.
    """
    if section == 'main':
        config.add_section(section)
    f = open ("/etc/livecreator.conf","w")
    config.write(f)
    f.close()


def WriteNewOptionConfig(config, option):
    """
        If "option" element is missing in config file
        write it in the main section from config file.
    """
    #hoption= {'rootdir':'/var/lib/livecreator/',
              #'templatedir':'/var/lib/livecreator/templates/',
              #'tmpdir':'/var/lib/livecreator/tmp/',
              #'tmpdir':'/var/lib/livecreator/tmp/',
              #'imgdir':'/var/lib/livecreator/img/',
              #'hookdir':'/var/lib/livecreator/hook/',
              #'pkgdir':'/var/lib/livecreator/packages/',
              #'depurl':'http://ftp.debian.org/debian',
              #'secudepurl':'http://security.debian.org',
              #'ftpproxy':'',
              #'httpproxy':'',
              #'version':"0.3.178"}
    hoption = allPreferences()

    for optionname in hoption.keys():
        if optionname == option:
            config.set("main", optionname, hoption[optionname])
    f = open ("/etc/livecreator.conf","w")
    config.write(f)
    f.close()


def writeNewConfig():
    """
        Create a new config file with default values.
    """
    #hoption= {'rootdir':'/var/lib/livecreator/',
              #'templatedir':'/var/lib/livecreator/templates/',
              #'tmpdir':'/var/lib/livecreator/tmp/',
              #'imgdir':'/var/lib/livecreator/img/',
              #'hookdir':'/var/lib/livecreator/hook/',
              #'pkgdir':'/var/lib/livecreator/packages/',
              #'depurl':'http://ftp.debian.org/debian',
              #'secudepurl':'http://security.debian.org',
              #'ftpproxy':'',
              #'httpproxy':'',
              #'version':"0.3.178"}
    hoption = allPreferences()

    newconfig = ConfigParser.ConfigParser()
    newconfig.add_section("main")
    for option in hoption.keys():
        newconfig.set("main", option, hoption[option])
    f = open ("/etc/livecreator.conf","w")
    newconfig.write(f)
    f.close()
    return newconfig


def writeOptionValues(config,optionsList):
    """
        Write options changed by the interface.
    """
    for option in optionsList:
        config.set("main",option,optionsList[option])
    f = open ("/etc/livecreator.conf","w")
    config.write(f)
    f.close()


def cleanWorkSpace(config):
    """
        Clean work space :
         - remove temp folder
         - umount tmp FS
    """
    templivecreatorDir = config.get("main","tmpdir")+"/tmp_livecreator"
    umountDir = ["/chroot/dev/pts", "/chroot/sys", "/chroot/proc"]
    # Unmount
    cmd = "umount"
    for folder in umountDir:
         cmd += " %s/%s"%(templivecreatorDir,folder)
    umountCmd = sp.Popen (cmd,
                     bufsize = 1,
                     shell = True,
                     stdout = sp.PIPE,
                     stderr = sp.STDOUT)
    try:
        umountCmd.stdout.readlines()
    except Exception, e:
        cleanWorkSpace(config)
    # Remove tmp folder
    cmd = "rm -rf %s/"%templivecreatorDir
    umountCmd2 = sp.Popen (cmd,
                    bufsize = 1,
                    shell = True,
                    stdout = sp.PIPE,
                    stderr = sp.STDOUT)
    try:
        umountCmd2.stdout.readlines()
    except Exception, e:
        cleanWorkSpace(config)
