#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import sys, shutil, os
from PyQt4 import QtCore, QtGui
from livecreator_misc import functions
from livecreator_misc import tips
from livecreator_qt import filedialog


class CreatePage(QtGui.QWidget):
    def __init__(self, livecreatorConfig, xmlFile, parent=None):
        """
            this class defines the template management page
        """
        QtGui.QWidget.__init__(self, parent)
        # Load config and xml file
        self.livecreatorConfig = livecreatorConfig
        self.xmlFile = xmlFile
        #Buttons
        self.newTemplate = QtGui.QPushButton( _("New") )
        self.newTemplate.setToolTip(tips.allTips().template_new)
        addTemplate = QtGui.QPushButton( _("Import") )
        addTemplate.setToolTip(tips.allTips().template_import)
        self.loadTemplate = QtGui.QPushButton( _("Load/view"))
        self.loadTemplate.setToolTip(tips.allTips().template_load)
        self.editTemplate = QtGui.QPushButton( _("Edit") )
        self.editTemplate.setToolTip(tips.allTips().template_edit)
        self.delTemplateButton = QtGui.QPushButton( _("Delete") )
        self.delTemplateButton.setToolTip(tips.allTips().template_del)
        self.delTemplateButton.setDisabled(1)
        buttonsGroup = QtGui.QGroupBox(self.tr( _("Action") ))
        #Add Buttons in layout
        buttonsLayout = QtGui.QVBoxLayout()
        buttonsLayout.addWidget( self.newTemplate,1)
        buttonsLayout.addWidget(addTemplate,1)
        buttonsLayout.addWidget(self.editTemplate)
        buttonsLayout.addWidget(self.loadTemplate)
        buttonsLayout.addWidget(self.delTemplateButton)
        buttonsGroup.setLayout(buttonsLayout)
        #list template
        templateGroup = QtGui.QGroupBox(self.tr( _("Template choice") ))
        templateLayout = QtGui.QVBoxLayout()
        self.templateList = QtGui.QListWidget()
        templateLayout.addWidget(self.templateList)
        templateGroup.setLayout(templateLayout)
        # Description template
        nametemplateLabel = QtGui.QLabel( _("Template name: ") )
        self.nametemplateEdit = QtGui.QLineEdit()
        self.nametemplateEdit.setDisabled(1)
        hostnameLabel = QtGui.QLabel( _("Hostname: ") )
        self.hostnameEdit = QtGui.QLineEdit()
        self.hostnameEdit.setDisabled(1)
        usernameLabel = QtGui.QLabel( _("Login: ") )
        self.usernameEdit = QtGui.QLineEdit()
        self.usernameEdit.setDisabled(1)
        langLabel = QtGui.QLabel( _("Langage: ") )
        self.langEdit = QtGui.QLineEdit()
        self.langEdit.setDisabled(1)
        kbdLabel = QtGui.QLabel( _("Keyboard: ") )
        self.kbdEdit = QtGui.QLineEdit()
        self.kbdEdit.setDisabled(1)
        typeLabel = QtGui.QLabel( _("Live type: ") )
        self.typeEdit = QtGui.QLineEdit()
        self.typeEdit.setDisabled(1)
        archLabel = QtGui.QLabel( _("Architecture: ") )
        self.archEdit = QtGui.QLineEdit()
        self.archEdit.setDisabled(1)
        imgbootLabel = QtGui.QLabel( _("Boot picture: ") )
        self.imgbootEdit = QtGui.QLineEdit()
        self.imgbootEdit.setDisabled(1)
        cryptLabel = QtGui.QLabel( _("Encryption type: ") )
        self.cryptEdit = QtGui.QLineEdit()
        self.cryptEdit.setDisabled(1)
        timeoutLabel = QtGui.QLabel( _("Time before boot: ") )
        self.timeoutEdit = QtGui.QLineEdit()
        self.timeoutEdit.setDisabled(1)
        persistentLabel = QtGui.QLabel( _("Persistence: ") )
        self.persistentEdit = QtGui.QLineEdit()
        self.persistentEdit.setDisabled(1)
        automountLabel = QtGui.QLabel( _("Partitions automount: ") )
        self.automountEdit = QtGui.QLineEdit()
        self.automountEdit.setDisabled(1)
        dhcpLabel = QtGui.QLabel( _("Automatic network configuration (DHCP): ") )
        self.dhcpEdit = QtGui.QLineEdit()
        self.dhcpEdit.setDisabled(1)
        imgloadLabel = QtGui.QLabel( _("Bootsplash picture: ") )
        self.imgloadEdit = QtGui.QLineEdit()
        self.imgloadEdit.setDisabled(1)
        commentLabel = QtGui.QLabel( _("Comments: ") )
        self.commentEdit = QtGui.QTextEdit()
        self.commentEdit.setReadOnly(1)
        # Description layout
        descriptionGroup = QtGui.QGroupBox(self.tr( _("Template description: ") ))
        descriptionLayout = QtGui.QGridLayout()
        descriptionLayout.addWidget(self.kbdEdit)
        descriptionGroup.setLayout(descriptionLayout)
        descriptionGroup.adjustSize()
        ## Generals options of description layout
        descriptionLayout.addWidget(nametemplateLabel,0,0)
        descriptionLayout.addWidget(self.nametemplateEdit,0,1,1,1)
        descriptionLayout.addWidget(hostnameLabel,1,0)
        descriptionLayout.addWidget(self.hostnameEdit,1,1,1,1)
        descriptionLayout.addWidget(usernameLabel,2,0)
        descriptionLayout.addWidget(self.usernameEdit,2,1,1,1)
        descriptionLayout.addWidget(langLabel,3,0)
        descriptionLayout.addWidget(self.langEdit,3,1,1,1)
        descriptionLayout.addWidget(kbdLabel,4,0)
        descriptionLayout.addWidget(self.kbdEdit,4,1,1,1)
        descriptionLayout.addWidget(typeLabel,5,0)
        descriptionLayout.addWidget(self.typeEdit,5,1,1,1)
        #descriptionLayout.addWidget(archLabel,6,0)
        #descriptionLayout.addWidget(self.archEdit,6,1,1,1)
        descriptionLayout.addWidget(imgbootLabel,7,0)
        descriptionLayout.addWidget(self.imgbootEdit,7,1,1,1)
        ## Advanced options of description layout
        descriptionLayout.addWidget(cryptLabel,8,0)
        descriptionLayout.addWidget(self.cryptEdit,8,1,1,1)
        descriptionLayout.addWidget(timeoutLabel,9,0)
        descriptionLayout.addWidget(self.timeoutEdit,9,1,1,1)
        #descriptionLayout.addWidget(persistentLabel,10,0)
        #descriptionLayout.addWidget(self.persistentEdit,10,1,1,1)
        descriptionLayout.addWidget(automountLabel,11,0)
        descriptionLayout.addWidget(self.automountEdit,11,1,1,1)
        descriptionLayout.addWidget(dhcpLabel,12,0)
        descriptionLayout.addWidget(self.dhcpEdit,12,1,1,1)
        descriptionLayout.addWidget(imgloadLabel,13,0)
        descriptionLayout.addWidget(self.imgloadEdit,13,1,1,1)
        descriptionLayout.addWidget(commentLabel,14,0,1,2)
        descriptionLayout.addWidget(self.commentEdit,15,0,1,2)
        # Main layout
        mainLayout = QtGui.QHBoxLayout()
        mainLayout.addWidget(buttonsGroup,1)
        mainLayout.addWidget(templateGroup,2)
        mainLayout.addWidget(descriptionGroup,3)
        mainLayout.addSpacing(12)
        self.setLayout(mainLayout)
        # Fill available template list
        self.fillList()

        #Signals
        ## Description signal
        QtCore.QObject.connect(self.templateList,
                            QtCore.SIGNAL('itemSelectionChanged()'),self.showDescription )
        ## Import template signal
        QtCore.QObject.connect(addTemplate ,QtCore.SIGNAL('clicked()'),self.xmlBrowse )
        ## Del template signal
        QtCore.QObject.connect(self.delTemplateButton ,QtCore.SIGNAL('clicked()'),self.delTemplate )


    def xmlBrowse(self):
        """
            Import a template
        """
        # Load template folder
        templateDir = self.livecreatorConfig.get("main","templatedir")
        # Open file window 
        filenames = filedialog.fileOpenDialog.getOpenFileNames(self, self.tr( _("Import a template") ),
                                   templateDir,"Xml (*.xml);;All Files (*.*)" )
        for file in filenames:
            # Take file name
            filename= os.path.basename(str(file))
            try:
                if not filename.endswith(".xml"):
                    # Copy file and change its extension to ".xml"
                    shutil.copy(str(file),"%s/%s.xml"%(templateDir,filename))
                else:
                    shutil.copy(str(file),"%s/%s"%(templateDir,filename))
            except Exception, e:
                     self.fillList()
        self.fillList()


    def delTemplate(self):
        """
            Delete a template in template list and in template folder
        """
        # Load template folder
        templateDir = self.livecreatorConfig.get("main","templatedir")
        if self.templateList.selectedItems():
            # Item seleted
            itemSeleted = self.templateList.currentItem().text()
            # Validation box
            question = QtGui.QMessageBox( _("Question"),
                _("Delete template %s ?") %itemSeleted,
                QtGui.QMessageBox.Question, 
                QtGui.QMessageBox.Yes, 
                QtGui.QMessageBox.No, 
                0 )
            question.setButtonText(QtGui.QMessageBox.Yes, _("&Yes") )
            question.setButtonText(QtGui.QMessageBox.No,_("&No") )
            if question.exec_() == QtGui.QMessageBox.Yes:
                # Delete from template folder
                os.remove( "%s/%s.xml"%(templateDir, itemSeleted) )
                # reload template list
                self.fillList()
                # clean
                self.showDescription()

    def fillList(self):
        """
            Fill template list
        """
        self.templateList.clear()
        fileList = functions.readDir(self.livecreatorConfig.get("main","templatedir"),"*.xml")
        fileList.replaceInStrings(".xml","")
        self.templateList.addItems(fileList)


    def showDescription(self):
        """
            Show description of selected template
        """
        descOptions = { 'desc_hostname' : self.hostnameEdit,
                'desc_username' : self.usernameEdit,
                'desc_lang' : self.langEdit,
                'desc_kbd' : self.kbdEdit,
                'desc_type' : self.typeEdit,
                'desc_arch' : self.archEdit,
                'desc_bootsplash' : self.imgbootEdit,
                'desc_crypt_level' : self.cryptEdit,
                'desc_timeout' : self.timeoutEdit,
                'desc_persistence' : self.persistentEdit,
                'desc_automount' : self.automountEdit,
                'desc_dhcp' : self.dhcpEdit,
                'desc_usplash' : self.imgloadEdit,
                'desc_comment' : self.commentEdit,
            }
        # If a item in template list is selected
        if self.templateList.selectedItems():
            # Enable delete template button
            self.delTemplateButton.setDisabled(0)
            # Take selectedtemplate name
            templatename = self.templateList.selectedItems()[0].text()
            # Load template folder
            xmlpath = "%s"%(self.livecreatorConfig.get("main", "templatedir"))
            # Load template from template folder
            self.xmlFile.loadXML("%s.xml"%templatename, xmlpath)
            # Write template name in description
            self.nametemplateEdit.setText(templatename)
            for key in descOptions.keys():
                # Write all options in description
                descOptions[key].setText(QtCore.QString.fromUtf8(self.xmlFile.getXMLValue(key).encode("iso8859-15") ) )
        else:
            # If no template is selected then delete button is disabled
            self.delTemplateButton.setDisabled(1)
            # clear description fields
            self.nametemplateEdit.clear()
            for key in descOptions.keys():
                descOptions[key].clear()
