#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import sys
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4 import QtCore, QtGui
from customtabs import gencustomtab
from customtabs import hookcustomtab
from customtabs import advcustomtab
from customtabs import desccustomtab
from livecreator_misc import tips

class CustomizePage(QTabWidget):
    def __init__(self, livecreatorConfig, xmlFile, parent = None):
        QTabWidget.__init__(self, parent)
        """
            This class defines the page which contains
            all tabs to cumstomize the Live
        """
        # Load config and xml file
        self.livecreatorConfig = livecreatorConfig
        self.xmlFile = xmlFile
        # Create tabs
        self.genCustomTab = gencustomtab.GenCustomize(self.livecreatorConfig, self.xmlFile)
        self.hookCustomTab = hookcustomtab.HookCustomize(self.livecreatorConfig, self.xmlFile)
        self.advCustomTab = advcustomtab.AdvCustomize(self.livecreatorConfig, self.xmlFile)
        self.descCustomTab = desccustomtab.DescCustomize(self.livecreatorConfig, self.xmlFile)
        # Add tabs to the page
        self.addTab(self.descCustomTab , _("&Template description"))
        self.addTab(self.genCustomTab, _("&General options"))
        self.addTab(self.advCustomTab , _("&Avanced options"))
        self.addTab(self.hookCustomTab, _("&Personalization scripts"))
        self.setTabToolTip(3,tips.allTips().custom_hook)
        # Signals to update description fields when a
        # field change in any tab
        QtCore.QObject.connect(self.genCustomTab.hostnameEdit, QtCore.SIGNAL('textChanged(QString)'), self.sendOptions )
        QtCore.QObject.connect(self.genCustomTab.userEdit, QtCore.SIGNAL('textChanged(QString)'), self.sendOptions )
        QtCore.QObject.connect(self.genCustomTab.langCombo, QtCore.SIGNAL('currentIndexChanged(QString)'), self.sendOptions )
        QtCore.QObject.connect(self.genCustomTab.kbdCombo, QtCore.SIGNAL('currentIndexChanged(QString)'), self.sendOptions )
        QtCore.QObject.connect(self.genCustomTab.typeCombo, QtCore.SIGNAL('currentIndexChanged(QString)'), self.sendOptions )
        QtCore.QObject.connect(self.genCustomTab.editImage, QtCore.SIGNAL('textChanged(QString)'), self.sendOptions )
        QtCore.QObject.connect(self.advCustomTab.cryptCombo, QtCore.SIGNAL('currentIndexChanged(QString)'), self.sendOptions )
        QtCore.QObject.connect(self.advCustomTab.timeoutEdit, QtCore.SIGNAL('textChanged(QString)'), self.sendOptions )
        QtCore.QObject.connect(self.advCustomTab.persitentCombo, QtCore.SIGNAL('currentIndexChanged(QString)'), self.sendOptions )
        QtCore.QObject.connect(self.advCustomTab.automountCombo, QtCore.SIGNAL('currentIndexChanged(QString)'), self.sendOptions )
        QtCore.QObject.connect(self.advCustomTab.DHCPCombo, QtCore.SIGNAL('currentIndexChanged(QString)'), self.sendOptions )
        QtCore.QObject.connect(self.advCustomTab.imageloadEdit, QtCore.SIGNAL('textChanged(QString)'), self.sendOptions )
        QtCore.QObject.connect(self.advCustomTab.archCombo, QtCore.SIGNAL('currentIndexChanged(QString)'), self.sendOptions )
        # Disable options for option mode
        QtCore.QObject.connect(self.advCustomTab.interModeCombo ,QtCore.SIGNAL('currentIndexChanged (QString)'),
                                    self.interativeMode )
        # disable image loading for crypt
        QtCore.QObject.connect(self.advCustomTab.cryptCombo,
                                QtCore.SIGNAL('currentIndexChanged (QString)'),
                                self.loadingImage )
        # disable image loading for automount
        QtCore.QObject.connect(self.advCustomTab.automountCombo,
                                QtCore.SIGNAL('currentIndexChanged (QString)'),
                                self.loadingImage )
        # disable image loading for DHCP
        QtCore.QObject.connect(self.advCustomTab.DHCPCombo,
                                QtCore.SIGNAL('currentIndexChanged (QString)'),
                                self.loadingImage )
        # disable image loading for language
        QtCore.QObject.connect(self.genCustomTab.langCombo,
                                QtCore.SIGNAL('currentIndexChanged (QString)'),
                                self.loadingImage )
        # disable image loading for keyboard
        QtCore.QObject.connect(self.genCustomTab.kbdCombo,
                                QtCore.SIGNAL('currentIndexChanged (QString)'),
                                self.loadingImage )


    def sendOptions(self):
        """
            Get Options set in the interface (does not get packages)
        """
        # Clear all description fields
        self.descCustomTab.clearDescFields()
        optionsList = []
        #get all general options
        optionsList.append(('hostname', self.genCustomTab.hostnameEdit.text()))
        optionsList.append(('username', self.genCustomTab.userEdit.text()))
        #optionsList.append(('descriptionText', self.genCustomTab.descriptionText.toHtml()))
        optionsList.append(('lang', self.genCustomTab.langCombo.currentText()))
        optionsList.append(('kbd', self.genCustomTab.kbdCombo.currentText()))
        optionsList.append(('type', self.genCustomTab.typeCombo.currentText()))
        optionsList.append(('bootsplash', self.genCustomTab.editImage.text()))
        #get all advanced options
        optionsList.append(('crypt', self.advCustomTab.cryptCombo.currentText()))
        optionsList.append(('timeout', self.advCustomTab.timeoutEdit.text()))
        optionsList.append(('persistence', self.advCustomTab.persitentCombo.currentText()))
        optionsList.append(('automount', self.advCustomTab.automountCombo.currentText()))
        optionsList.append(('dhcp', self.advCustomTab.DHCPCombo.currentText()))
        optionsList.append(('usplash', self.advCustomTab.imageloadEdit.text()))
        optionsList.append(('arch', self.advCustomTab.archCombo.currentText()))
        # Write in description fields
        self.descCustomTab.initFields(optionsList)


    def interativeMode(self):
        """
            Enable or not interactive mode
        """
        # List of fielfs to disable in interactive mode
        listCombo = [   self.advCustomTab.automountCombo,
                        self.advCustomTab.DHCPCombo,
                        self.genCustomTab.langCombo,
                        self.genCustomTab.kbdCombo]
        # If interactive mode is activated
        if self.advCustomTab.interModeCombo.currentText() == "On":
            # Disable loading image
            self.advCustomTab.imageloadEdit.clear()
            self.advCustomTab.imageloadEdit.setDisabled(1)
            self.advCustomTab.addimageload.setDisabled(1)
            # Disable the others fields
            for combo in listCombo:
                # Count nomber of item and select the last one
                combo.setCurrentIndex(combo.count()-1)
                # disable the combo box
                combo.setDisabled(1) 
        # If interactive mode is not activated
        if self.advCustomTab.interModeCombo.currentText() == "Off":
            # Enable the others fields
            for combo in listCombo:
                # Select the first item ("Off")
                combo.setCurrentIndex(0)
                # Enable the combo box
                combo.setDisabled(0)
        # If no anwser asked while booting then we
        # can enable loading image
        self.disableLoadingImage()



    def loadingImage(self, mode):
        """
            Enable or not loading image infunction of differents options
        """
        # If crypt mode OR dhcp mode OR automount mode OR
        # language mode OR keyboard are equal
        # to "choose while booting"
        # then we disable loading image
        # If all of them are NOT equal to "choose while booting"
        # then we enable the loading image
        #is not activated and crypt Mode is not activated 
        if mode != "Off":
            # Disable loading image
            self.advCustomTab.imageloadEdit.clear()
            self.advCustomTab.imageloadEdit.setDisabled(1)
            self.advCustomTab.addimageload.setDisabled(1)
        # If no anwser asked while booting then we
        # can enable loading image
        self.disableLoadingImage()


    def disableLoadingImage(self):
        if( self.advCustomTab.interModeCombo.currentText() == "Off" and
            self.advCustomTab.automountCombo.currentText() != _("Choose during live boot") and
            self.advCustomTab.DHCPCombo.currentText() != _("Choose during live boot") and
            self.advCustomTab.cryptCombo.currentText() == "Off" and
            self.genCustomTab.langCombo.currentText() != _("Choose during live boot") and
            self.genCustomTab.kbdCombo.currentText() != _("Choose during live boot")
            ):
            # Enable loading image
            self.advCustomTab.addimageload.setDisabled(0)
            self.advCustomTab.imageloadEdit.setDisabled(0)
