#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import sys
from PyQt4 import QtCore, QtGui
from livecreator_misc import tips

class AdvCustomize(QtGui.QWidget):
    def __init__(self,livecreatorConfig, xmlFile, parent=None):
        QtGui.QWidget.__init__(self, parent)

        self.livecreatorConfig = livecreatorConfig
        self.xmlFile = xmlFile

        cryptGroup = QtGui.QGroupBox( _("Encryption type") )
        cryptlayout = QtGui.QGridLayout()
        self.cryptCombo = QtGui.QComboBox()
        self.cryptCombo.addItem(u"Off")
        self.cryptCombo.addItem(u"aes128")
        self.cryptCombo.addItem(u"aes192")
        self.cryptCombo.addItem(u"aes256")
        self.cryptText = QtGui.QLabel()
        self.cryptText.setText( _('<font color="red">Password will be asked during generation</font>') )
        self.cryptText.setVisible(0)
        cryptlayout.addWidget(self.cryptCombo,0,0,0,5)
        cryptlayout.addWidget(self.cryptText,1,0)
        cryptGroup.setLayout(cryptlayout)
        cryptGroup.setToolTip(tips.allTips().custom_adv_crypt)

        timeoutGroup = QtGui.QGroupBox( _("Time before boot (in seconds)") )
        timeoutlayout = QtGui.QVBoxLayout()
        self.timeoutEdit = QtGui.QLineEdit()
        timeoutlayout.addWidget(self.timeoutEdit)
        timeoutGroup.setLayout(timeoutlayout)
        timeoutGroup.setToolTip(tips.allTips().custom_adv_timeout)

        persitentGroup = QtGui.QGroupBox( _("Persistence") )
        persitentGroup.hide()
        persitentlayout = QtGui.QVBoxLayout()
        self.persitentCombo = QtGui.QComboBox()
        self.persitentCombo.addItem(u"Off")
        self.persitentCombo.addItem(u"On")
        persitentlayout.addWidget(self.persitentCombo)
        persitentGroup.setLayout(persitentlayout)

        automountGroup = QtGui.QGroupBox( _("Partitions automount") )
        automountlayout = QtGui.QVBoxLayout()
        self.automountCombo = QtGui.QComboBox()
        self.automountCombo.addItem(u"Off")
        self.automountCombo.addItem(u"On")
        self.automountCombo.addItem( _("Forensic") )
        self.automountCombo.addItem( _("Choose during Live boot") )
        automountlayout.addWidget(self.automountCombo)
        automountGroup.setLayout(automountlayout)
        automountGroup.setToolTip(tips.allTips().custom_adv_automount)

        DHCPGroup = QtGui.QGroupBox( _("Automatic network demand (DHCP)") )
        DHCPlayout = QtGui.QVBoxLayout()
        self.DHCPCombo = QtGui.QComboBox()
        self.DHCPCombo.addItem(u"Off")
        self.DHCPCombo.addItem(u"On")
        self.DHCPCombo.addItem( _("Choose during Live boot") )
        DHCPlayout.addWidget(self.DHCPCombo)
        DHCPGroup.setLayout(DHCPlayout)
        DHCPGroup.setToolTip(tips.allTips().custom_adv_dhcp)

        imageloadGroup = QtGui.QGroupBox( _("Bootsplash picture") )
        imageloadlayout = QtGui.QGridLayout()
        self.addimageload = QtGui.QPushButton( _("Browse...") )
        self.imageloadEdit = QtGui.QLineEdit()
        imageloadlayout.addWidget(self.addimageload,0,1)
        imageloadlayout.addWidget(self.imageloadEdit,0,0)
        imageloadGroup.setLayout(imageloadlayout)
        imageloadGroup.setToolTip(tips.allTips().custom_adv_usplash)

        #TODO: Disable this choice, by default we can only build same arch than host
        archGroup = QtGui.QGroupBox( _("Architecture") )
        archGroup.hide()
        archlayout = QtGui.QVBoxLayout()
        self.archCombo = QtGui.QComboBox()
        self.archCombo.addItem(u"i386")
        self.archCombo.addItem(u"amd64")
        self.archCombo.addItem(u"powerpc")
        archlayout.addWidget(self.archCombo)
        archGroup.setLayout(archlayout)

        interModeGroup = QtGui.QGroupBox( _("Interactif mode") )
        interModelayout = QtGui.QVBoxLayout()
        self.interModeCombo = QtGui.QComboBox()
        self.interModeCombo.addItem(u"Off")
        self.interModeCombo.addItem(u"On")
        interModelayout.addWidget(self.interModeCombo)
        interModeGroup.setLayout(interModelayout)
        interModeGroup.setToolTip(tips.allTips().custom_adv_intermode)

        suppGroup = QtGui.QGroupBox( _("Additional options") )
        supplayout = QtGui.QVBoxLayout()
        self.suppEdit = QtGui.QLineEdit()
        supplayout.addWidget(self.suppEdit)
        suppGroup.setLayout(supplayout)
        suppGroup.setToolTip(tips.allTips().custom_adv_options)

        mainLayout = QtGui.QGridLayout()
        mainLayout.addWidget(imageloadGroup,0,0)
        mainLayout.addWidget(timeoutGroup,0,1)
        mainLayout.addWidget(cryptGroup,1,0)
        mainLayout.addWidget(interModeGroup,1,1)
        mainLayout.addWidget(DHCPGroup,2,1)
        mainLayout.addWidget(automountGroup,2,0)
        mainLayout.addWidget(persitentGroup,3,0)
        mainLayout.addWidget(archGroup,3,1)
        mainLayout.addWidget(suppGroup,4,0,1,2)
        #mainLayout.addSpacing(12)
        #mainLayout.addStretch(1)

        QtCore.QObject.connect(self.addimageload ,QtCore.SIGNAL('clicked()'),self.imageBrowse )
        QtCore.QObject.connect(self.cryptCombo,QtCore.SIGNAL('currentIndexChanged(int)'),self.cryptTextShow )

        self.setLayout(mainLayout)


    def clearAdvOptions(self):
        """
            Restore options to default values
        """
        self.imageloadEdit.setText("")
        self.timeoutEdit.setText("")
        self.suppEdit.setText("")
        self.archCombo.setCurrentIndex(0)
        self.cryptCombo.setCurrentIndex(0)
        self.persitentCombo.setCurrentIndex(0)
        self.interModeCombo.setCurrentIndex(0)
        self.automountCombo.setCurrentIndex(0)
        self.DHCPCombo.setCurrentIndex(0)



    def imageBrowse(self):
        file = QtGui.QFileDialog.getOpenFileName(self, _("Bootsplash picture import"),
                                                           QtCore.QDir.currentPath(),"Images (*.png);;All files (*)")
        if file:
            self.imageloadEdit.setText(file)


    def initAdvOptions(self):
        """
            Initialize advanced options from xml file
        """
        # Text files
        self.timeoutEdit.setText(self.xmlFile.getXMLValue("timeout") )
        self.imageloadEdit.setText(self.xmlFile.getXMLValue("usplash") )
        self.suppEdit.setText(self.xmlFile.getXMLValue("supp") )

        # ComboBoxes
        index = self.cryptCombo.findText(self.xmlFile.getXMLValue("crypt_level"))
        if index >= 0:
            self.cryptCombo.setCurrentIndex(index)
        index = self.persitentCombo.findText(self.xmlFile.getXMLValue("persistence"))
        if index >= 0:
            self.persitentCombo.setCurrentIndex(index)
        index = self.automountCombo.findText(self.xmlFile.getXMLValue("automount"))
        if index >= 0:
            self.automountCombo.setCurrentIndex(index)
        index = self.DHCPCombo.findText(self.xmlFile.getXMLValue("dhcp"))
        if index >= 0:
            self.DHCPCombo.setCurrentIndex(index)
        index = self.interModeCombo.findText(self.xmlFile.getXMLValue("interactivemode"))
        if index >= 0:
            self.interModeCombo.setCurrentIndex(index)
        index = self.archCombo.findText(self.xmlFile.getXMLValue("architecture"))
        if index >= 0:
            self.archCombo.setCurrentIndex(index)


    def cryptTextShow(self,index):
        """
            show or not the crypt warning text
        """
        if index == 0 :
            self.cryptText.setVisible(0)
        else:
            self.cryptText.setVisible(1)
