#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import sys
from PyQt4 import QtCore, QtGui, Qt
from livecreator_misc import tips

class GenCustomize(QtGui.QWidget):
    def __init__(self,livecreatorConfig, xmlFile, parent=None):
        QtGui.QWidget.__init__(self, parent)

        self.livecreatorConfig = livecreatorConfig
        self.xmlFile = xmlFile

        hostnameGroup = QtGui.QGroupBox( _("Hostname") )
        hostnamelayout = QtGui.QVBoxLayout()
        self.hostnameEdit = QtGui.QLineEdit()
        self.hostnameEdit.setMaxLength(15)
        hostnamelayout.addWidget(self.hostnameEdit)
        hostnameGroup.setLayout(hostnamelayout)
        hostnameGroup.setToolTip(tips.allTips().custom_gen_hostname)

        userGroup = QtGui.QGroupBox( _("Login") )
        userlayout = QtGui.QVBoxLayout()
        self.userEdit = QtGui.QLineEdit()
        userlayout.addWidget(self.userEdit)
        userGroup.setLayout(userlayout)
        userGroup.setToolTip(tips.allTips().custom_gen_user)

        #descriptionGroup = QtGui.QGroupBox(u"Description"))
        #descriptionLayout = QtGui.QVBoxLayout()
        #self.descriptionText = QtGui.QTextEdit()
        #descriptionLayout.addWidget(self.descriptionText)
        #descriptionGroup.setLayout(descriptionLayout)

        langGroup = QtGui.QGroupBox( _("Langage") )
        langlayout = QtGui.QVBoxLayout()
        self.langCombo = QtGui.QComboBox()
        # List which contain of locales ( mask and data for combo box)
        langList = []
        # Read from file /usr/share/i18n/SUPPORTED
        # Processing locale to get language from locales
        localesFile = QtCore.QFile("/usr/share/i18n/SUPPORTED")
        localesFile.open(QtCore.QIODevice.ReadOnly)
        locale = localesFile.readLineData(localesFile.size()) 
        while locale:
            # Take locale only if in UTF8
            if locale.find("UTF-8") != -1 and locale.find("@") == -1:
                # Create temp locate to find language and country
                QCountry = QtCore.QLocale(locale[0:-1])
                # Create QString temp variable to store country and language
                countryLang = QtCore.QString("%s (%s)"%( QtCore.QLocale.countryToString(QCountry.country()),
                                QtCore.QLocale.languageToString(QCountry.language()) ) )
                # DO NOT STORE default 
                if not str(countryLang).startswith("Default"):
                    # Store each Utf8-8 locale in langList
                    # exemple of one element :
                    # [ "FRANCE (french)", "fr_FR.UTF-8 UTF-8" ]
                    langList.append( (  countryLang ,Qt.QVariant(locale[0:-1])   ) )
            # Read next line
            locale = localesFile.readLineData(localesFile.size()) 
        #Sort lang list by country
        langList.sort()
        # Fill the combo box
        for lang in langList:
            self.langCombo.addItem(lang[0],lang[1])
            # French language is selected by default
            # TODO: Changing "France" by te current system locale
            if str(lang[0]).startswith("France"):
                self.langCombo.setCurrentIndex(self.langCombo.count()-1)

        self.langCombo.addItem( _("Choose during Live boot") )
        langlayout.addWidget(self.langCombo)
        langGroup.setLayout(langlayout)
        langGroup.setToolTip(tips.allTips().custom_gen_lang)

        kbdGroup = QtGui.QGroupBox( _("Layout (keyboard)") )
        kbdlayout = QtGui.QVBoxLayout()
        self.kbdCombo = QtGui.QComboBox()

        # List which contains keymaps ( mask and data for combo box)
        keymapList = []
        # Read keymaps folder
        qKeymapDir = QtCore.QDir("/usr/share/keymaps/i386")
        # Test if the keymaps folder exists
        if qKeymapDir.exists():
            keymapsDir = QtCore.QDirIterator(qKeymapDir,QtCore.QDirIterator.Subdirectories)
            # List all files in keymaps folder
            while keymapsDir.hasNext():
                # Read file name
                keymap = str(keymapsDir.fileName())
                # Don't read "." and ".." folder
                if keymap == "." or keymap == ".." :
                    keymapsDir.next()
                else:
                    # Test if keymap lenght is superior to 3
                    if len(keymap) >= 3:
                        # keymap beginning by a language ( exemple : fr-latin9, us-latin1, ...)
                        if keymap[2] == "-" :
                            # delete ".kmap.gz" from the keymap name
                            keymap = keymap[0:-8]
                            # The 2 first letter correspond to a language
                            qLanguage = QtCore.QLocale(keymap[0:2])
                            language = str(QtCore.QLocale.languageToString(qLanguage.language()))
                            # If language is equal to C then language = default
                            if language == "C":
                                language = "Default"
                            mask = "%s (%s)"%( language, keymap )
                            keymapList.append( (mask , Qt.QVariant(keymap) ) )
                    keymapsDir.next()

        # If the keymap folder doesn't exist then we write 
        # a permanent list in the combo box
        else:
            keymapList.append(( "Bulgarian (bg-cp1251)", Qt.QVariant("bg-cp1251") ))
            keymapList.append(( "Byelorussian (be-latin1)", Qt.QVariant("be-latin1") ))
            keymapList.append(( "Default (br-abnt2)", Qt.QVariant("br-abnt2") ))
            keymapList.append(( "Default (br-latin1)", Qt.QVariant("br-latin1") ))
            keymapList.append(( "Default (cz-lat2)", Qt.QVariant("cz-lat2") ))
            keymapList.append(( "Default (cz-lat2-prog)", Qt.QVariant("cz-lat2-prog") ))
            keymapList.append(( "Default (cz-us-qwerty)", Qt.QVariant("cz-us-qwerty") ))
            keymapList.append(( "Default (cz-us-qwertz)", Qt.QVariant("cz-us-qwertz") ))
            keymapList.append(( "Default (cz-us-qwertz)", Qt.QVariant("cz-us-qwertz") ))
            keymapList.append(( "Default (dk-latin1)", Qt.QVariant("dk-latin1") ))
            keymapList.append(( "Default (gr-pc)", Qt.QVariant("gr-pc") ))
            keymapList.append(( "Default (gr-utf8)", Qt.QVariant("gr-utf8") ))
            keymapList.append(( "Default (il-heb)", Qt.QVariant("il-heb") ))
            keymapList.append(( "Default (il-phonetic)", Qt.QVariant("il-phonetic") ))
            keymapList.append(( "Default (la-latin1)", Qt.QVariant("la-latin1") ))
            keymapList.append(( "Default (pc-dvorak-latin1)", Qt.QVariant("pc-dvorak-latin1") ))
            keymapList.append(( "Default (se-fi-ir209)", Qt.QVariant("se-fi-ir209") ))
            keymapList.append(( "Default (se-fi-lat6)", Qt.QVariant("se-fi-lat6") ))
            keymapList.append(( "Default (se-ir209)", Qt.QVariant("se-ir209") ))
            keymapList.append(( "Default (se-lat6)", Qt.QVariant("se-lat6") ))
            keymapList.append(( "Default (se-latin1)", Qt.QVariant("se-latin1") ))
            keymapList.append(( "Default (sg-latin1)", Qt.QVariant("sg-latin1") ))
            keymapList.append(( "Default (sg-latin1-lk450)", Qt.QVariant("sg-latin1-lk450") ))
            keymapList.append(( "Default (ua-utf)", Qt.QVariant("ua-utf") ))
            keymapList.append(( "Default (ua-utf-ws)", Qt.QVariant("ua-utf-ws") ))
            keymapList.append(( "Default (ua-ws)", Qt.QVariant("ua-ws") ))
            keymapList.append(( "Default (us-intl.iso01)", Qt.QVariant("us-intl.iso01") ))
            keymapList.append(( "Default (us-intl.iso15)", Qt.QVariant("us-intl.iso15") ))
            keymapList.append(( "Default (us-latin1)", Qt.QVariant("us-latin1") ))
            keymapList.append(( "Estonian (et-nodeadkeys)", Qt.QVariant("et-nodeadkeys") ))
            keymapList.append(( "Finnish (fi-latin1)", Qt.QVariant("fi-latin1") ))
            keymapList.append(( "French (fr-latin0)", Qt.QVariant("fr-latin0") ))
            keymapList.append(( "French (fr-latin1)", Qt.QVariant("fr-latin1") ))
            keymapList.append(( "French (fr-latin9)", Qt.QVariant("fr-latin9") ))
            keymapList.append(( "French (fr-pc)", Qt.QVariant("fr-pc") ))
            keymapList.append(( "French (fr-x11)", Qt.QVariant("fr-x11") ))
            keymapList.append(( "German (de-latin1)", Qt.QVariant("de-latin1") ))
            keymapList.append(( "German (de-latin1-nodeadkeys)", Qt.QVariant("de-latin1-nodeadkeys") ))
            keymapList.append(( "Icelandic (is-latin1)", Qt.QVariant("is-latin1") ))
            keymapList.append(( "Icelandic (is-latin1)", Qt.QVariant("is-latin1") ))
            keymapList.append(( "Icelandic (is-latin1-us)", Qt.QVariant("is-latin1-us") ))
            keymapList.append(( "Italian (it-ibm)", Qt.QVariant("it-ibm") ))
            keymapList.append(( "Latvian (lv-latin4)", Qt.QVariant("lv-latin4") ))
            keymapList.append(( "Latvian (lv-latin7)", Qt.QVariant("lv-latin7") ))
            keymapList.append(( "Norwegian (no-latin1)", Qt.QVariant("no-latin1") ))
            keymapList.append(( "Norwegian (no-standard)", Qt.QVariant("no-standard") ))
            keymapList.append(( "Polish (pl-qwertz)", Qt.QVariant("pl-qwertz") ))
            keymapList.append(( "Portuguese (pt-latin1)", Qt.QVariant("pt-latin1") ))
            keymapList.append(( "Portuguese (pt-old)", Qt.QVariant("pt-old") ))
            keymapList.append(( "Romanian (ro-academic)", Qt.QVariant("ro-academic") ))
            keymapList.append(( "Romanian (ro-comma)", Qt.QVariant("ro-comma") ))
            keymapList.append(( "Russian (ru-cp1251)", Qt.QVariant("ru-cp1251") ))
            keymapList.append(( "Russian (ru-ms)", Qt.QVariant("ru-ms") ))
            keymapList.append(( "Russian (ru-yawerty)", Qt.QVariant("ru-yawerty") ))
            keymapList.append(( "Serbian (sr-cy)", Qt.QVariant("sr-cy") ))
            keymapList.append(( "Slovak (sk-prog)", Qt.QVariant("sk-prog") ))
            keymapList.append(( "Slovak (sk-prog-qwerty)", Qt.QVariant("sk-prog-qwerty") ))
            keymapList.append(( "Slovak (sk-prog-qwertz)", Qt.QVariant("sk-prog-qwertz") ))
            keymapList.append(( "Slovak (sk-qwerty)", Qt.QVariant("sk-qwerty") ))
            keymapList.append(( "Slovak (sk-qwertz)", Qt.QVariant("sk-qwertz") ))
            keymapList.append(( "Spanish (es-cp850)", Qt.QVariant("es-cp850") ))
            keymapList.append(( "Thai (th-tis)", Qt.QVariant("th-tis") ))

        #Sort lang list by language
        keymapList.sort()
        #set(keymapList)
        # Fill the combo box
        for keymap in keymapList:
            self.kbdCombo.addItem(keymap[0],keymap[1])
            # French language is selected by default
            #TODO: Pass this to system layout instead of arbitrary french layout
            if keymap[0] == "French (fr-latin9)":
                self.kbdCombo.setCurrentIndex(self.kbdCombo.count()-1)

        self.kbdCombo.addItem( _("Choose during Live boot"))
        kbdlayout.addWidget(self.kbdCombo)
        kbdGroup.setLayout(kbdlayout)
        kbdGroup.setToolTip(tips.allTips().custom_gen_kbd)

        typeGroup = QtGui.QGroupBox( _("Live type") )
        typelayout = QtGui.QVBoxLayout()
        self.typeCombo = QtGui.QComboBox()
        self.typeCombo.addItem(u"iso")
        self.typeCombo.addItem(u"usb-hdd")
        #self.typeCombo.addItem(u"net")
        typelayout.addWidget(self.typeCombo)
        typeGroup.setLayout(typelayout)
        typeGroup.setToolTip(tips.allTips().custom_gen_type)

        imageGroup = QtGui.QGroupBox( _("Boot loader picture") )
        imagelayout = QtGui.QGridLayout()
        addImage = QtGui.QPushButton( _("Browse...") )
        self.editImage = QtGui.QLineEdit()
        imagelayout.addWidget(addImage,0,1)
        imagelayout.addWidget(self.editImage,0,0)
        imageGroup.setLayout(imagelayout)
        imageGroup.setToolTip(tips.allTips().custom_gen_bootsplash)


        mainLayout = QtGui.QGridLayout()
        mainLayout.addWidget(hostnameGroup,0,0)
        mainLayout.addWidget(userGroup,0,1)
        mainLayout.addWidget(kbdGroup,2,1)
        mainLayout.addWidget(langGroup,2,0)
        mainLayout.addWidget(typeGroup,3,0)
        mainLayout.addWidget(imageGroup,3,1)
        #mainLayout.addSpacing(12)
        #mainLayout.addStretch(1)

        QtCore.QObject.connect(addImage ,QtCore.SIGNAL('clicked()'),self.imageBrowse )

        self.setLayout(mainLayout)


    def clearGenOptions(self):
        """
            Restaure options to default values
        """
        self.hostnameEdit.setText("")
        self.userEdit.setText("")
        self.editImage.setText("")
        self.typeCombo.setCurrentIndex(0)
        # French language is selected by default
        #TODO: Change this by the system language
        langSelectedIndex = self.langCombo.findText("France", QtCore.Qt.MatchStartsWith)
        self.langCombo.setCurrentIndex(langSelectedIndex)
        # French language is selected by default
        #TODO: Change this by the system layout
        keymapSelectedIndex = self.kbdCombo.findText("French (fr-latin9)", QtCore.Qt.MatchStartsWith)
        self.kbdCombo.setCurrentIndex(keymapSelectedIndex)


    def initGenOptions(self):
        """
            Initialize general options from xml file
        """
        # Clear Options
        self.clearGenOptions()
        # Text files
        self.hostnameEdit.setText(self.xmlFile.getXMLValue("hostname") )
        self.userEdit.setText(self.xmlFile.getXMLValue("username") )
        self.editImage.setText(self.xmlFile.getXMLValue("bootsplash") )

        # ComboBoxes
        index = self.langCombo.findText(self.xmlFile.getXMLValue("lang"))
        if index >= 0:
            self.langCombo.setCurrentIndex(index)
        index = self.kbdCombo.findText(self.xmlFile.getXMLValue("kbd"))
        if index >= 0:
            self.kbdCombo.setCurrentIndex(index)
        index = self.typeCombo.findText(self.xmlFile.getXMLValue("type"))
        if index >= 0:
            self.typeCombo.setCurrentIndex(index)


    def imageBrowse(self):
        file = QtGui.QFileDialog.getOpenFileName(self, _("Boot picture import"),
                         QtCore.QDir.currentPath(),
                         "Images(*.png);;All files (*)")
        if file:
            self.editImage.setText(file)
