#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys, os
from PyQt4 import QtCore, QtGui
from intro import intropage
from create import createpage
from pkgset import pkgsetpage
from customize import customizepage
from summary import summarypage
from testimg import testimgpage
from pref import preferencepage
from about import aboutpage
from livecreator_xml import xmltemplate as tpl
from livecreator_misc import tips
from livecreator_qt import filedialog

# Icons
#import icons_rc

class FrontEnd(QtGui.QDialog):
    def __init__(self,livecreatorConfig, parent=None):
        QtGui.QDialog.__init__(self, parent)

        # Set size window
        self.setMinimumSize(800, 460)
        self.resize(800, 460)

        # Set icon window
        self.setWindowIcon(QtGui.QIcon("images/livecreator_icon.png"))

        self.livecreatorConfig = livecreatorConfig
        self.xmlFile = tpl.xmlTemplate()

        self.contentsWidget = QtGui.QListWidget()
        self.pagesWidget = QtGui.QStackedWidget()

        self.contentsWidget.setViewMode(QtGui.QListView.IconMode)
        self.contentsWidget.setIconSize(QtCore.QSize(32,32))
        self.contentsWidget.setMovement(QtGui.QListView.Static)
        self.contentsWidget.setMaximumWidth(128)
        self.contentsWidget.setResizeMode(QtGui.QListView.Adjust)
        self.contentsWidget.setFlow(QtGui.QListView.TopToBottom);
        self.contentsWidget.setWrapping(0);
        self.contentsWidget.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.contentsWidget.setFixedWidth(155)

        # Load tabs
        self.introductionPage = intropage.IntroductionPage(self.livecreatorConfig)
        self.createPage = createpage.CreatePage(self.livecreatorConfig, self.xmlFile)
        self.pkgSetPage = pkgsetpage.PkgSetPage(self.livecreatorConfig, self.xmlFile)
        self.customizePage = customizepage.CustomizePage(self.livecreatorConfig, self.xmlFile)
        self.testImgPage = testimgpage.TestImgPage(self.livecreatorConfig)
        # testImgPage is passed as parameter
        # in order to permit the images list refresh
        # after an image generation
        self.summaryPage = summarypage.SummaryPage(self.livecreatorConfig,self.xmlFile , self.testImgPage)
        self.prefPage = preferencepage.PreferencePage(self.livecreatorConfig)
        self.aboutPage = aboutpage.AboutPage(self.livecreatorConfig)
        # Create Tabs
        self.pagesWidget.addWidget(self.introductionPage)
        self.pagesWidget.addWidget(self.createPage)
        self.pagesWidget.addWidget(self.pkgSetPage)
        self.pagesWidget.addWidget(self.customizePage)
        self.pagesWidget.addWidget(self.summaryPage)
        self.pagesWidget.addWidget(self.testImgPage)
        self.pagesWidget.addWidget(self.prefPage)
        self.pagesWidget.addWidget(self.aboutPage)

        # Buttons
        # Previous, Next buttons
        self.PVbuttonsLayout = QtGui.QGridLayout()
        self.nextButton = QtGui.QPushButton( _("Next") )
        self.nextButton.setIcon(QtGui.QIcon("images/next.png"))
        self.nextButton.setFixedWidth(100)
        self.prevButton = QtGui.QPushButton( _("Previous") )
        self.prevButton.setIcon(QtGui.QIcon("images/previous.png"))
        self.prevButton.setFixedWidth(100)
        self.PVbuttonsLayout.addWidget(self.prevButton,0,1)
        self.PVbuttonsLayout.addWidget(self.nextButton,0,2)
        self.PVbuttonsLayout.setColumnStretch (0,1)
        self.PVbuttonsLayout.setColumnStretch (3,1)
        # Save , Exit Buttons
        self.saveButton = QtGui.QPushButton(self.tr( _("Save") ))
        self.saveButton.setIcon(QtGui.QIcon("images/save.png"))
        self.saveButton.setDisabled(1)
        self.saveButton.setToolTip(tips.allTips().mainqt_save)
        closeButton = QtGui.QPushButton(self.tr( _("Close") ) )
        closeButton.setIcon(QtGui.QIcon("images/exit.png"))
        closeButton.setToolTip(tips.allTips().mainqt_exit)
        self.buttonsLayout = QtGui.QHBoxLayout()
        self.buttonsLayout.addStretch(1)
        self.buttonsLayout.addWidget(self.saveButton)
        self.buttonsLayout.addWidget(closeButton)

        separator = QtGui.QFrame()
        separator.setFrameShape(QtGui.QFrame.HLine)

        self.createIcons()
        self.contentsWidget.setCurrentRow(0)

        mainLayout = QtGui.QGridLayout()
        mainLayout.addWidget(self.contentsWidget,0,0,4,1)
        mainLayout.addWidget(self.pagesWidget,0,1)
        mainLayout.addLayout(self.PVbuttonsLayout,1,1 )
        mainLayout.addWidget(separator,2,1)
        mainLayout.addLayout(self.buttonsLayout,3,1 )

        self.setLayout(mainLayout)
        self.setWindowTitle(self.tr("LiveCreator - %s"%self.livecreatorConfig.get("main","version")))

        self.connect(self.nextButton, 
                QtCore.SIGNAL("clicked()"), self.nextPageButton)
        self.connect(self.createPage.templateList, 
                QtCore.SIGNAL("itemDoubleClicked(QListWidgetItem *)"), self.nextPageButton)
        self.connect(self.prevButton, 
                QtCore.SIGNAL("clicked()"), self.previousPageButton)
        self.connect(closeButton, 
                QtCore.SIGNAL("clicked()"), self.close )
        self.connect(self.saveButton, 
                QtCore.SIGNAL("clicked()"), self.saveXmlValues)
        self.connect(self.createPage.newTemplate,
                QtCore.SIGNAL("clicked()"), self.newTemplate)
        self.connect(self.createPage.loadTemplate,
                QtCore.SIGNAL("clicked()"), self.endLoadTemplate)
        self.connect(self.createPage.editTemplate,
                QtCore.SIGNAL("clicked()"), self.editTemplate)


    def reject(self):
        """
            Catch escape button event
            and fdo nothing
        """


    def closeEvent(self, event):
        """
           Question the user and exit if he answers "yes"
        """
        if self.summaryPage.procxterm.state() == QtCore.QProcess.NotRunning:
            question = QtGui.QMessageBox( _("Quit software"),
                    _("You want quit software. Do you want save the template??"),
                    QtGui.QMessageBox.Information, 
                    QtGui.QMessageBox.Yes,
                    QtGui.QMessageBox.Ignore,
                    QtGui.QMessageBox.No, 
                    )
            question.setButtonText(QtGui.QMessageBox.Yes, _("&Save changes") )
            question.setButtonText(QtGui.QMessageBox.No, _("&Discard changes") )
            question.setButtonText(QtGui.QMessageBox.Ignore, _("&Cancel") )
            answer = question.exec_()
            if answer == QtGui.QMessageBox.Yes:
                # Exit with save
                if self.saveXmlValues() == False:
                    event.ignore()
                    self.closeEvent(event)
                else:
                    event.accept()
            elif answer == QtGui.QMessageBox.No:
                # Exit without save
                event.accept()
            else:
                event.ignore()
        else:
            event.ignore()


    def nextPageButton(self):
        """
            Permit to change page with next button
        """
        currentIndex = self.pagesWidget.currentIndex()
         # Check the selection profil list
        if currentIndex == 1:
            if self.createPage.templateList.selectedItems():
                self.editTemplate()
            #else:
                #self.newTemplate()
        # Change page works with that because 
        # the next line change list selection
        # and a signal is emit when the selection 
        # is changed. this signal is connected
        # with changePage function
        self.contentsWidget.setCurrentRow(currentIndex+1)


    def previousPageButton(self):
        """
            Permit to change page with previous button
        """
        currentIndex = self.pagesWidget.currentIndex()
        # Change page works with that because 
        # the next line change list selection
        # and a signal is emit when the selection 
        # is changed. this signal is connected
        # with changePage function
        self.contentsWidget.setCurrentRow(currentIndex-1)


    def newTemplate(self):
        """
            Create a new empty template
        """
        self.contentsWidget.setCurrentRow(2)
        self.pagesWidget.setCurrentIndex(2)
        title = _("LiveCreator - %s - New template - No name") %self.livecreatorConfig.get("main","version") 
        self.setWindowTitle(QtCore.QString(title))
        self.cleanAllFields()


    def editTemplate(self):
        """
            Load xml values from xml file
            and go to package tab
        """
        if self.createPage.templateList.selectedItems():
            # go to the next page
            self.contentsWidget.setCurrentRow(2)
            self.pagesWidget.setCurrentIndex(2)
            # Set windows title
            templatename = self.createPage.templateList.selectedItems()[0].text()
            title =  _("LiveCreator - %(version)s - Template edition - %(template)s ") %{'version': self.livecreatorConfig.get("main","version"), 'template': templatename}
            self.setWindowTitle(QtCore.QString(title))
            # clean old values in the interface
            self.cleanAllFields()
            # load values from xml file
            self.LoadXmlvalues(templatename)
            # Unselect item in the template list
            item = self.createPage.templateList.selectedItems()
            self.createPage.templateList.setItemSelected(item[0],0)


    def endLoadTemplate(self):
        """
            Load xml values from xml file
            and go to load/see tab
        """
        if self.createPage.templateList.selectedItems():
            templatename = self.createPage.templateList.selectedItems()[0].text()
            title =  _("LiveCreator - %(version)s - Loading template - %(template)s") %{'version': self.livecreatorConfig.get("main","version"), 'template': templatename}
            self.setWindowTitle(QtCore.QString(title )  )
            # clean old values in the interface
            self.cleanAllFields()
            # load values from xml file
            self.LoadXmlvalues(templatename) #setCurrentRow
            self.summaryPage.generateSummary(self.getAllOptions())
            self.contentsWidget.setCurrentRow(4)
            self.pagesWidget.setCurrentIndex(4)


    def cleanAllFields(self):
        """
            Clean all fields to prepare a new template
        """
        # Clean local package list
        self.pkgSetPage.summaryPkgTab.localPkgList.clear()
        self.pkgSetPage.localPkgTab.pkgselectList.clear()
        self.pkgSetPage.localPkgTab.packageList.clear()
        self.pkgSetPage.localPkgTab.fillList()
########
########
########
        # Clean local package list
        self.pkgSetPage.summaryPkgTab.sourceList.clear()
        self.pkgSetPage.sourcePkgTab.sourceList.clear()
        self.pkgSetPage.sourcePkgTab.sourceSelectedList.clear()
        self.pkgSetPage.sourcePkgTab.fillList()
########
########
########
        # Clean upstream package list
        self.pkgSetPage.summaryPkgTab.upstreamPkgList.clear()
        self.pkgSetPage.upstreamPkgTab.cleanUpstreamPkgList()
        # Clean hook scripts lists
        self.customizePage.hookCustomTab.posthookList.clear()
        self.customizePage.hookCustomTab.prehookList.clear()
        self.customizePage.hookCustomTab.hookList.clear()
        self.customizePage.hookCustomTab.fillList()
        # Clean descriptionText
        self.customizePage.sendOptions()
        # Clean generals options
        self.customizePage.genCustomTab.clearGenOptions()
        # Clean advanced options
        self.customizePage.advCustomTab.clearAdvOptions()


    def LoadXmlvalues(self,templatename):
        """
            Load xml values from xml file
        """
        # Load Xml file
        xmlpath = "%s"%(self.livecreatorConfig.get("main", "templatedir"))
        self.xmlFile.loadXML("%s.xml"%templatename, xmlpath)

        # Write local package list
        self.pkgSetPage.summaryPkgTab.loadLocalPkgList()
        self.pkgSetPage.localPkgTab.initSelectedpkgList()
        # Write upstream package list
        self.pkgSetPage.loadUpstreamPkgList()
########
########
########
        # Write source package list
        self.pkgSetPage.sourcePkgTab.initSelectedpkgList()
        self.pkgSetPage.summaryPkgTab.loadSourcePkgList()
########
########
########
        # Write general options
        self.customizePage.genCustomTab.initGenOptions()
        # Write Advanced options
        self.customizePage.advCustomTab.initAdvOptions()
        # Write selected Hook script
        self.customizePage.hookCustomTab.initHookSelected()
        # Write Description form
        self.customizePage.descCustomTab.initDescription()


    def getAllOptions(self):
        """
            Get All Options set in the interface
            Typical option : ("Name", "Value", "XmlName", "LiveHelperOption / LiveCreator Option")
            example :        ("Nom de l'utilisateur", "user1", "username", "--username")
        """
        allOptionsList = []
        # Get all general options
        allOptionsList.append(( _("Hostname"), self.customizePage.genCustomTab.hostnameEdit.text(),'hostname', "--hostname" ) )
        allOptionsList.append(( _("Login"), self.customizePage.genCustomTab.userEdit.text() ,'username', "--username"  ))
        # For language, we made a mask to show country (language) and to select locale to create Lives
        langSelectedIndex = self.customizePage.genCustomTab.langCombo.currentIndex()
        allOptionsList.append(( _("Language"), self.customizePage.genCustomTab.langCombo.currentText() ,'lang', "LANG", self.customizePage.genCustomTab.langCombo.itemData(langSelectedIndex).toString()  ))
        # For keyboard, we made a mask to show language (layoutà and to select layout to create Lives
        layoutSelectedIndex = self.customizePage.genCustomTab.kbdCombo.currentIndex()
        allOptionsList.append(( _("Layout (keyboard)"), self.customizePage.genCustomTab.kbdCombo.currentText(), 'kbd', "BOOTAPPEND_KBD"  , self.customizePage.genCustomTab.kbdCombo.itemData(layoutSelectedIndex).toString() ))
        allOptionsList.append(( _("Live type"), self.customizePage.genCustomTab.typeCombo.currentText(), 'type', "--binary-images"))
        allOptionsList.append(( _("Boot picture"), self.customizePage.genCustomTab.editImage.text(), 'bootsplash', "--syslinux-splash"))
        # Get hook scripts lists
        allOptionsList.append(( _("Inside chroot scripts list"), self.customizePage.hookCustomTab.prehookList, 'hookchroot', "HOOKS_CHROOT") )
        allOptionsList.append(( _("Outside chroot scripts list"), self.customizePage.hookCustomTab.posthookList, 'hookbinary', "HOOKS_BINARY"))
        # Get all advanced options
        allOptionsList.append(( _("Encryption type"), self.customizePage.advCustomTab.cryptCombo.currentText(), 'crypt_level', "ENCRYPTION"))
        allOptionsList.append(( _("Time before boot"), self.customizePage.advCustomTab.timeoutEdit.text(), 'timeout', "--syslinux-timeout" ))
        allOptionsList.append(( _("Persistence"), self.customizePage.advCustomTab.persitentCombo.currentText(), 'persistence', "PERSISTENCE"))
        allOptionsList.append(( _("Partitions automount"), self.customizePage.advCustomTab.automountCombo.currentText(), 'automount', "AUTOMOUNT"))
        allOptionsList.append(( _("Automatic netwwork (DHCP)"), self.customizePage.advCustomTab.DHCPCombo.currentText(),'dhcp', "DHCP"))
        allOptionsList.append(( _("Splashboot"), self.customizePage.advCustomTab.imageloadEdit.text(), 'usplash', "USPLASH" ))
        allOptionsList.append(( _("Interactive mode"), self.customizePage.advCustomTab.interModeCombo.currentText(), 'interactivemode', "INTERACTIVE_MODE" ))
        allOptionsList.append(( _("Architecture"), self.customizePage.advCustomTab.archCombo.currentText(), 'architecture', "--architecture" ))
        allOptionsList.append(( _("Manual options"), self.customizePage.advCustomTab.suppEdit.text(), 'supp', "SUPP" ))

        # Get packages lists
        allOptionsList.append(( _("Distribution packages list"),
                self.pkgSetPage.summaryPkgTab.upstreamPkgList, 'upstream', "--packages"))
        allOptionsList.append(( _("Local package list"),
                self.pkgSetPage.summaryPkgTab.localPkgList, 'local', "LOCALDEBS"))
########
########
########
        allOptionsList.append(( _("Sources packages list"),
                self.pkgSetPage.summaryPkgTab.sourceList, 'source', "SOURCES_PACKAGES"))
########
########
########
        # Get Description values
        allOptionsList.append(('desc_hostname', self.customizePage.descCustomTab.hostnameEdit.text(), 'desc_hostname'))
        allOptionsList.append(('desc_username', self.customizePage.descCustomTab.usernameEdit.text(), 'desc_username'))
        allOptionsList.append(('desc_lang', self.customizePage.descCustomTab.langEdit.text(), 'desc_lang'))
        allOptionsList.append(('desc_kbd', self.customizePage.descCustomTab.kbdEdit.text(), 'desc_kbd'))
        allOptionsList.append(('desc_type', self.customizePage.descCustomTab.typeEdit.text(), 'desc_type'))
        allOptionsList.append(('desc_arch', self.customizePage.descCustomTab.archEdit.text(), 'desc_arch'))
        allOptionsList.append(('desc_bootsplash', self.customizePage.descCustomTab.imgbootEdit.text(), 'desc_bootsplash'))
        allOptionsList.append(('desc_crypt_level', self.customizePage.descCustomTab.cryptEdit.text(), 'desc_crypt_level'))
        allOptionsList.append(('desc_timeout', self.customizePage.descCustomTab.timeoutEdit.text(), 'desc_timeout'))
        allOptionsList.append(('desc_persistence', self.customizePage.descCustomTab.persistentEdit.text(), 'desc_persistence'))
        allOptionsList.append(('desc_automount', self.customizePage.descCustomTab.automountEdit.text(), 'desc_automount'))
        allOptionsList.append(('desc_dhcp', self.customizePage.descCustomTab.dhcpEdit.text(), 'desc_dhcp'))
        allOptionsList.append(('desc_usplash', self.customizePage.descCustomTab.imgloadEdit.text(), 'desc_usplash'))
        allOptionsList.append(('desc_comment', self.customizePage.descCustomTab.commentaireEdit.toPlainText(), 'desc_comment'))
        return allOptionsList


    def changePage(self, current, previous):
        """
            Change page with icon and manage next, provious
            and save bouton enability
        """
        if not current:
            current = previous
        self.pagesWidget.setCurrentIndex(self.contentsWidget.row(current))
        if self.pagesWidget.currentIndex() == 0:
            self.saveButton.setDisabled(1)
            self.prevButton.setDisabled(1)
            self.nextButton.setDisabled(0)
        if self.pagesWidget.currentIndex() == 1:
            self.saveButton.setDisabled(1)
            self.prevButton.setDisabled(0)
            self.nextButton.setDisabled(0)
        if self.pagesWidget.currentIndex() == 2:
            self.saveButton.setDisabled(0)
            self.prevButton.setDisabled(0)
            self.nextButton.setDisabled(0)
        if self.pagesWidget.currentIndex() == 3:
            self.saveButton.setDisabled(0)
            self.prevButton.setDisabled(0)
            self.nextButton.setDisabled(0)
        if self.pagesWidget.currentIndex() == 4:
            self.saveButton.setDisabled(0)
            self.prevButton.setDisabled(0)
            self.nextButton.setDisabled(0)
            #Load summary
            self.summaryPage.generateSummary(self.getAllOptions())
        if self.pagesWidget.currentIndex() == 5:
            self.saveButton.setDisabled(0)
            self.prevButton.setDisabled(0)
            self.nextButton.setDisabled(1)
        if self.pagesWidget.currentIndex() == 6:
            self.saveButton.setDisabled(1)
            self.prevButton.setDisabled(1)
            self.nextButton.setDisabled(1)
        if self.pagesWidget.currentIndex() == 7:
            self.saveButton.setDisabled(1)
            self.prevButton.setDisabled(1)
            self.nextButton.setDisabled(1)


    def createIcons(self):
	cellSize=QtCore.QSize(150,85)
        homeButton = QtGui.QListWidgetItem(self.contentsWidget)
        homeButton.setText( _("Home") )
        homeButton.setIcon(QtGui.QIcon("images/home.png"))
        homeButton.setTextAlignment(QtCore.Qt.AlignHCenter)
        homeButton.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
	homeButton.setSizeHint(cellSize)
        homeButton.setToolTip(tips.allTips().mainqt_intro)

        step1Button = QtGui.QListWidgetItem(self.contentsWidget)
        step1Button.setText( _("Templates") )
        step1Button.setIcon(QtGui.QIcon("images/step1.png"))
        step1Button.setTextAlignment(QtCore.Qt.AlignHCenter)
        step1Button.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
	step1Button.setSizeHint(cellSize)
        step1Button.setToolTip(tips.allTips().mainqt_template)

        step2Button = QtGui.QListWidgetItem(self.contentsWidget)
        step2Button.setText( _("Packages") )
        step2Button.setIcon(QtGui.QIcon("images/step2.png"))
        step2Button.setTextAlignment(QtCore.Qt.AlignHCenter)
        step2Button.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
	step2Button.setSizeHint(cellSize)
        step2Button.setToolTip(tips.allTips().mainqt_packages)

        step3Button = QtGui.QListWidgetItem(self.contentsWidget)
        step3Button.setText( _("Options") )
        step3Button.setIcon(QtGui.QIcon("images/step3.png"))
        step3Button.setTextAlignment(QtCore.Qt.AlignHCenter)
        step3Button.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
	step3Button.setSizeHint(cellSize)
        step3Button.setToolTip(tips.allTips().mainqt_custom)

        genButton = QtGui.QListWidgetItem(self.contentsWidget)
        genButton.setText( _("Generation") )
        genButton.setIcon(QtGui.QIcon("images/generation.png"))
        genButton.setTextAlignment(QtCore.Qt.AlignHCenter)
        genButton.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
	genButton.setSizeHint(cellSize)
        genButton.setToolTip(tips.allTips().mainqt_summary)

        testButton = QtGui.QListWidgetItem(self.contentsWidget)
        testButton.setText( _("Lives") )
        testButton.setIcon(QtGui.QIcon("images/test.png"))
        testButton.setTextAlignment(QtCore.Qt.AlignHCenter)
        testButton.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
	testButton.setSizeHint(cellSize)
        testButton.setToolTip(tips.allTips().mainqt_testimg)

        prefButton = QtGui.QListWidgetItem(self.contentsWidget)
        prefButton.setText( _("Settings") )
        prefButton.setIcon(QtGui.QIcon("images/preferences.png"))
        prefButton.setTextAlignment(QtCore.Qt.AlignHCenter)
        prefButton.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
	prefButton.setSizeHint(cellSize)
        prefButton.setToolTip(tips.allTips().mainqt_prefs)

        aboutButton = QtGui.QListWidgetItem(self.contentsWidget)
        aboutButton.setText( _("About") )
        aboutButton.setIcon(QtGui.QIcon("images/about.png"))
        aboutButton.setTextAlignment(QtCore.Qt.AlignHCenter)
        aboutButton.setFlags(QtCore.Qt.ItemIsSelectable | QtCore.Qt.ItemIsEnabled)
	aboutButton.setSizeHint(cellSize)
        aboutButton.setToolTip(tips.allTips().mainqt_about)

        self.connect(self.contentsWidget,
                QtCore.SIGNAL("currentItemChanged(QListWidgetItem *, QListWidgetItem *)"), self.changePage)


    def saveXmlValues(self):
        """
            Save xml file
        """
        # Box to browse where to save xml file
        templateDir = self.livecreatorConfig.get("main","templatedir")
        file = QtGui.QFileDialog.getSaveFileName(self,
                                    self.tr( _("Save Template") ),
                                    templateDir,"Xml (*.xml)" )
        if not file :
            return False
        else :
            # Collect all options
            allOptions = self.getAllOptions()
            # Open xml file where to save
            folder = os.path.dirname(str(file) )
            filename = os.path.basename(str(file) )
            #xmlpath = "%s"%(self.livecreatorConfig.get("main", "templatedir"))
            #self.xmlFile.loadXML("%s.xml"%filename, xmlpath)
            # Write template name
            if filename.endswith(".xml"):
                self.xmlFile.setXMLValue("templatename",filename[0:-4])
            else:
                self.xmlFile.setXMLValue("templatename",filename)
            # write options
            for options in allOptions:
                templist = []
                # options[1] is the value      (e.g. Off)
                # options[2] is the value name (e.g. DHCP)
                if type(options[1]) is type(QtCore.QString()) :
                    self.xmlFile.setXMLValue(options[2],str(options[1].toUtf8()))
                if type(options[1]) is type(QtGui.QListWidget()) :
                    # translate QListWidget to python list
                    for index in range(options[1].count()):
                        templist.append(str(options[1].item(index).text() ) )
                    #Save lists
                    if options[2] == 'upstream':
                        self.xmlFile.setPackagesList(templist, "upstream")
                    if options[2] == 'local':
                        self.xmlFile.setPackagesList(templist, "local")
                    if options[2] == 'hookchroot':
                        self.xmlFile.setHooksList(templist, "chroot")
                    if options[2] == 'hookbinary':
                        self.xmlFile.setHooksList(templist, "binary")
    ################
    ################
    ################
                    if options[2] == 'source':
                        self.xmlFile.setSourcesList(templist, "source")
    ################
    ################
    ################
            #print str(allOptions[0][1])
            #self.xmlFile.setXMLValue("hostname","live")
            if filename.endswith(".xml"):
                self.xmlFile.toXML(filename,folder)
            else:
                self.xmlFile.toXML("%s.xml"%filename,folder)
            self.createPage.fillList()
            # Change application title
            title =  _("LiveCreator - %(version)s - Template edition - %(template)s ") %{'version': self.livecreatorConfig.get("main","version"), 'template': filename}
            self.setWindowTitle(QtCore.QString(title))
