#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import sys
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyQt4 import QtCore, QtGui
from pkgsettabs import localpkgtab
from pkgsettabs import upstreampkgtab
from pkgsettabs import summarypkgtab
from pkgsettabs import sourcepkgtab
import shutil
from livecreator_misc import tips

class PkgSetPage(QTabWidget):
    def __init__(self, livecreatorConfig, xmlFile, parent = None):
        QTabWidget.__init__(self, parent)

        self.livecreatorConfig = livecreatorConfig
        self.xmlFile = xmlFile

        self.summaryPkgTab = summarypkgtab.SummaryPkg(self.livecreatorConfig, self.xmlFile)
        self.upstreamPkgTab = upstreampkgtab.UpstreamPkg(self.livecreatorConfig, self.xmlFile, self.summaryPkgTab)
        self.localPkgTab = localpkgtab.LocalPkg(self.livecreatorConfig, self.xmlFile)
        self.sourcePkgTab = sourcepkgtab.SourcePkg(self.livecreatorConfig, self.xmlFile)

        self.addTab(self.summaryPkgTab, _("Packages summary") )
        self.setTabToolTip(0,tips.allTips().packages_summary)
        self.addTab(self.upstreamPkgTab, _("&Official distribution packages") )
        self.setTabToolTip(1,tips.allTips().packages_upstream)
        self.addTab(self.localPkgTab, _("&Local packages") )
        self.setTabToolTip(2,tips.allTips().packages_local)
        self.addTab(self.sourcePkgTab, _("&Sources packages") )
        self.setTabToolTip(3,tips.allTips().packages_source)

        # Add package signals
        self.connect(self.localPkgTab.packageList, 
                QtCore.SIGNAL("itemDoubleClicked(QListWidgetItem *)"), 
                self.addSelectedPkg)
        QtCore.QObject.connect(self.localPkgTab.selectPackage ,
                QtCore.SIGNAL('clicked()'),self.addSelectedPkg )
        # remove package signals
        self.connect(self.localPkgTab.pkgselectList ,
                QtCore.SIGNAL("itemDoubleClicked(QListWidgetItem *)"),
                self.removePkg)
        QtCore.QObject.connect(self.localPkgTab.removePackage ,
                QtCore.SIGNAL('clicked()'),self.removePkg )

###################
###################
###################
        # Add package signals
        self.connect(self.sourcePkgTab.sourceList, 
                QtCore.SIGNAL("itemDoubleClicked(QListWidgetItem *)"), 
                self.addSource)
        QtCore.QObject.connect(self.sourcePkgTab.selectPackage ,
                QtCore.SIGNAL('clicked()'),self.addSource )
        # remove package signals
        self.connect(self.sourcePkgTab.sourceSelectedList ,
                QtCore.SIGNAL("itemDoubleClicked(QListWidgetItem *)"),
                self.removeSource)
        QtCore.QObject.connect(self.sourcePkgTab.removePackage ,
                QtCore.SIGNAL('clicked()'),self.removeSource )
###################
###################
###################

    def loadUpstreamPkgList(self):
        """
            Load upstream PkgList from xml
            and check them in upstream tab
            and add this in summary package tab
        """
        self.upstreamPkgTab.checkUpstreamPkgFromXml()
        self.summaryPkgTab.loadUpstreamPkgList()

    def addSelectedPkg(self):
        if self.localPkgTab.packageList.selectedItems():
            pkg = self.localPkgTab.packageList.takeItem(self.localPkgTab.packageList.currentRow())
            self.localPkgTab.pkgselectList.addItem(pkg)
            self.summaryPkgTab.addLocalPkgList(pkg)

    def removePkg(self):
        if self.localPkgTab.pkgselectList.selectedItems():
            pkgindex = self.localPkgTab.pkgselectList.currentRow()
            self.localPkgTab.packageList.addItem(self.localPkgTab.pkgselectList.takeItem(pkgindex))
            self.summaryPkgTab.delLocalPkgList(pkgindex)



###################
###################
###################

    def addSource(self):
        """
            move source package from available sources list to selected sources list
            (add source to the template)
        """
        if self.sourcePkgTab.sourceList.selectedItems():
            source =               self.sourcePkgTab.sourceList.takeItem(self.sourcePkgTab.sourceList.currentRow())
            self.sourcePkgTab.sourceSelectedList.addItem(source)
            self.summaryPkgTab.addSourcePkgList(source)


    def removeSource(self):
        """
            move source package from selected sources list to available sources list 
            (del source from the template)
        """
        if self.sourcePkgTab.sourceSelectedList.selectedItems():
            sourceindex = self.sourcePkgTab.sourceSelectedList.currentRow()
            self.sourcePkgTab.sourceList.addItem(self.sourcePkgTab.sourceSelectedList.takeItem(sourceindex))
            self.summaryPkgTab.delSourcePkgList(sourceindex)
###################
###################
###################
