#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys
from PyQt4 import QtCore, QtGui
import apt
import re
from PyQt4.QtCore import Qt

class PackDict():
    """
        All current distribution packages.
        Added fields: CheckedState, reversed dependances...
        This is a kind of apt.Cache with specific needs
    """
    def __init__(self, parent=None):
        """
            Apt Cache: obtain informations on packages
            Checked: selected by user or as dependance
            NeededBy: is a dependance of this package(s)
        """
        self.cache = {}
        for package in apt.Cache():
            self.cache[package.name]={}
            self.cache[package.name]["Package"] = package
            self.cache[package.name]["Checked"] = False
            self.cache[package.name]["NeededBy"] = set()


    def packageList(self):
        """
            Return the full list of avalaible packages.
        """
        return self.cache.keys()


    def package(self, packageName):
        """
            Return an "apt/package" object
        """
        return self.cache[packageName]["Package"]


    def packageChecked(self, packageName):
        """
            Return the package "choosed" state
        """
        return self.cache[packageName]["Checked"]


    def packageParents(self, packageName):
        """
            Return a list containing all the package which depends of this one.
        """
        return self.cache[packageName]["NeededBy"]



class UpstreamPkg(QtGui.QMainWindow):
    def __init__(self, livecreatorConfig, xmlFile, summaryPkgTab, parent=None):
        QtGui.QMainWindow.__init__(self, parent)

        self.livecreatorConfig = livecreatorConfig
        self.xmlFile = xmlFile
        # Used to pass and update the packages list :
        self.summaryPkgTab = summaryPkgTab

        #Widget
        tsize = QtGui.QLabel()
        description = QtGui.QTextEdit()
        filterLine = QtGui.QLineEdit()
        filterList = QtGui.QComboBox()

        view =  QtGui.QTableView()
        self.table = MyTableModel(self.xmlFile, tsize, description, filterLine, 
                                    filterList, self.summaryPkgTab, view)
        view.setModel(self.table)

        #Tablecostumieze
        #orien = QtGui.Qt.Orientation(1)
        h = view.horizontalHeader()
        h.setResizeMode(0, QtGui.QHeaderView.Fixed)
        h.setResizeMode(1, QtGui.QHeaderView.Interactive)
        h.setResizeMode(2, QtGui.QHeaderView.Interactive)
        h.setResizeMode(3, QtGui.QHeaderView.Interactive)
        h.setResizeMode(4, QtGui.QHeaderView.Stretch)
        h.resizeSection(0,30)
        h.resizeSection(1,250)
        h.resizeSection(3,200)
        h.setSortIndicatorShown(0)
        view.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        view.setSortingEnabled(1)

        ### Size
        #tsize = QtGui.QLabel()
        tsize.setText( _("Total size: OB") )
        #tsize.setText("test")

        ###Description
        #description = QtGui.QTextEdit()
        descriptionGrid = QtGui.QGridLayout()
        description.setReadOnly(1)
       # descriptionGrid.addWidget(description,1,1,1,2)

        #FilterLine
        #filterLine = QtGui.QLineEdit()
        filtergrid = QtGui.QGridLayout()
        #filterList = QtGui.QComboBox()
        for columnName in self.table.columns:
                columnHeader = self.table.columnHeaders.get(columnName, columnName)
                if columnHeader != "":
                    filterList.addItem(columnHeader)
        #TODO: add a filter for checked package
        #filterList.addItem('Nom')
        #filterList.setEditable(True)
        #filterList.setAutoCompletion(True)
        #filterList.showPopup()
        #filterLine = QtGui.QLineEdit()
        #filtergrid.addWidget(description,1,1,1,2)
        #description.sizePolicy().setHorizontalPolicy(QtGui.QSizePolicy.Expanding)
        filtergrid.addWidget(filterLine,1,1)
        filtergrid.addWidget(filterList,1,2)
        filterw = QtGui.QWidget()
        filterw.setLayout(filtergrid)
        filterw.setFixedHeight(40)
        #if filterLine.text():
            #temp = filterLine.text()
        #else:
            #temp = ''



        # Signals

        QtCore.QObject.connect(
                filterLine,QtCore.SIGNAL('textChanged(QString)'), 
                self.table.changedFilter  )
        QtCore.QObject.connect(
                filterList,QtCore.SIGNAL('currentIndexChanged(QString)'), 
                self.table.changedFilter  )
        QtCore.QObject.connect(
                view ,QtCore.SIGNAL('clicked(QModelIndex)'),
                self.table.checkbox )
        QtCore.QObject.connect(
                view ,QtCore.SIGNAL('clicked(QModelIndex)'),
                self.table.description )
        QtCore.QObject.connect(
                view ,QtCore.SIGNAL('clicked(QModelIndex)'),
                self.table.tsize )

        # Set interface
        self.setCentralWidget(view)

        # Set Dock widget
        dockFilt = QtGui.QDockWidget( _("Search") )
        dockFilt.setWidget(filterw)
        dockFilt.setAllowedAreas(Qt.BottomDockWidgetArea | Qt.TopDockWidgetArea)
        dockFilt.setFeatures(QtGui.QDockWidget.NoDockWidgetFeatures | QtGui.QDockWidget.DockWidgetMovable)
        self.addDockWidget(Qt.TopDockWidgetArea,dockFilt)

        dockDesc = QtGui.QDockWidget( _("Description") )
        dockDesc.setWidget(description)
        dockDesc.setAllowedAreas(Qt.BottomDockWidgetArea | Qt.TopDockWidgetArea)
        dockDesc.setFeatures(QtGui.QDockWidget.NoDockWidgetFeatures | QtGui.QDockWidget.DockWidgetMovable)
        self.addDockWidget(Qt.BottomDockWidgetArea,dockDesc)
        #dock = QtGui.QDockWidget(u"Description")
        #dock.setWidget(filterw)
        #self.addDockWidget(Qt.BottomDockWidgetArea,dock)
        #mainLayout = QtGui.QGridLayout()
        #mainLayout.addWidget(tsize,2,1)
        #mainLayout.addWidget(view,1,1)
        #mainLayout.addWidget(filterw,3,1)

        ##mainLayout.addWidget(packageGroup)
        ###mainLayout.addSpacing(12)
        ###mainLayout.addStretch(1)


        #self.setLayout(mainLayout)

    def checkUpstreamPkgFromXml(self):
        """
            Initialize local pagckages list
        """
        self.table.initList(self.xmlFile.getPackagesList("upstream"))


    def cleanUpstreamPkgList(self):
        """
            Initialize local pagckages list
        """
        self.table.cleanCheckList()
        self.table.initPackDict()



class MyTableModel(QtCore.QAbstractTableModel):
    """
        Table containing package list.
    """
    def __init__(self, xmlFile,  tSize, Description, FilterLine, FilterList, summaryPkgTab, parent=None):
        """
            TableModel contains a PackDict which is the list of avalaible packages
            (list build from host apt sources), and for each package a state :
             -> Checked : if user choose package or if it's a dependance
             -> NeededBy : list of other packages that depend of this one. 
        """
        QtCore.QAbstractTableModel.__init__(self, parent)

        self.tSize = tSize
        self.Description = Description
        self.FilterLine = FilterLine
        self.FilterList = FilterList
        self.view = parent
        self.xmlFile = xmlFile
        self.summaryPkgTab = summaryPkgTab

        # This constructor can take a while : he build a dictionnary with all
        # the packages.
        self.packdict = PackDict()
        
        ### FIXME FROM HERE (A)
        self.cache = apt.Cache()
        self.fulllist = self.cache.keys()
        self.list = self.fulllist[:]
        
        #self.checks = {}
         # ajout (initialisation) du champ "checked" dans chaque instance de Package du cache
        self.cleanCheckList() 
        #for package in self.cache:
        #    package.checked = False
            #print package.priority
            #if package.priority == self.cache['libc6'].priority:
            #    print "pass"
            #    package.checked = True
        #for package in self.cache:
            #package.neededBy = set()
        ### FIXME TO HERE (A)

        self.columns = ['checked', 'name', 'packageSize', 'candidateVersion', 'summary']
        self.columnHeaders = { 'checked': '',
                                'name': _("Package name"),
                                'packageSize' : _("Size"),
                                'candidateVersion': _("Version"),
                                'summary' : _("summary"),
                                }

        #Load upstream package from xml file
        package = self.packdict.package("vim")
        deps=set()
        dependances = self.dependencies(self.cache,package,deps,"Depends")
        print ""
        print ""
        print ""
        print dependances
        print ""
        print ""
        print ""
        print package.candidate.dependencies
        print ""
        print ""
        print ""


########
### FIXME FROM HERE (B)
    def cleanCheckList(self):
        """
            Uncheck all packages in list and
            surcharge package objects
        """
        print "passe"
#        for package in self.cache:
#            package.checked = False
        for package in self.cache:
            package.neededBy = set()
### FIXME TO HERE (B)

    def initList(self,pkglist):
        """
            Initialize the choosed package list from xml file
        """
        for pack in pkglist:
            try :
                package = self.cache[pack]
            except Exception,e:
                continue
            package.checked = not package.checked
            deps = set()
            deps = self.dependencies(self.cache,package,deps,"Depends")
            preDeps = set()
            preDeps = self.dependencies(self.cache,package, preDeps, "PreDepends")
            #print preDeps
            #print deps
            for dep in preDeps:
                deps.add(dep)
            for deppackname in deps:
                deppack = self.cache[deppackname]
                if package.checked:
                    deppack.neededBy.add(package)
                else:
                    deppack.neededBy.remove(package)
                try:
                    row = self.list.index(deppackname)
                    self.emit(QtCore.SIGNAL('dataChanged(QModelIndex,QModelIndex)'),
                        self.index(row,0), self.index(row, len(self.columns)-1))
                except ValueError: pass
        return True

######
    def changedFilter(self):
        #print type(self.FilterLine)
        # appelé lorsque la combobox ou la zone de saisie du filtre change.
        # ... d'abord, on récupère la valeur de la combobox
        filterCat = self.FilterList.currentText()
        # ... on trouve le nom de la colonne correspondante
        reverseMap = dict( [ (QtCore.QString(v),k) for (k,v) in self.columnHeaders.items() ] )
        filterCat = reverseMap[filterCat]
        # ... on récupère la valeur de la zone de saisie du filtre
        #filter = filterLine.text()
        filter = unicode(self.FilterLine.text())
        #print repr(filter)
        self.view.model().emit(QtCore.SIGNAL('layoutAboutToBeChanged()'))
        if filter:
            #filter = unicode(filterLine.text())
            # ... on annonce �|  la vue que le modèle va être modifié
            # ... on compile une expression régulière
            matcher = re.compile(filter)
            # ... on effectue la recherche �|  proprement parler
            self.list = [pkgName for pkgName
                in self.fulllist
                if matcher.search(getattr(self.cache[pkgName],filterCat))]
            # ... on annonce �|  la vue qu'on a fini de modifier le modèle
        else:
            self.list = self.fulllist[:]
        self.view.model().emit(QtCore.SIGNAL('layoutChanged()'))


    def package_checkState(self, index):
        """
            Give the information on the checkbox status
        """
        package = self.getPackageByIndex(index)
        if package.checked or package.neededBy:
            return QtCore.Qt.Checked
        else:
            return QtCore.Qt.Unchecked


    def getPackageByRow(self, row):
        return self.cache[self.list[row]]


    def getPackageByIndex(self, index):
        return self.getPackageByRow(index.row())


    def rowCount(self, parent=None):
        return len(self.list)


    def columnCount(self, parent=None):
        return len(self.columns)


    def formatColumn(self, columnName, value):
        if columnName=='checked':
            return ''
        if columnName=='summary':
            #return value.decode('utf-8')
            return unicode(value.decode('utf-8'))
        if "size" in columnName.lower():
            return self.formatSize(value)
#            try:
#                size=int(value)
#                if size > 1024:
#                    if size/1024 > 1024:
#                        return "%d Mo"%(size/1024/1024)
#                    return "%d Ko"%(size/1024)
#                return "%d Octets"%(size)
#            except:
#                return None
        return value


    def formatSize(self, value):
        try:
            size=int(value)
            if size == 0:
                return "0 B"
            if size > 1024:
                if size/1024 > 1024:
                    return "%d M"%(size/1024/1024)
                return "%d K"%(size/1024)
            return "%d B"%(size)
        except:
            return None


    def data(self, index, role):
        if not index.isValid(): return QtCore.QVariant()
        package = self.getPackageByIndex(index)

        if role == QtCore.Qt.DisplayRole:
            attrName = self.columns[index.column()]
            try: attrValue = getattr(package,attrName)
            except: attrValue = None
            formattedValue = self.formatColumn(attrName,attrValue)
            if formattedValue == None: return QtCore.QVariant()
            return QtCore.QVariant(formattedValue)

        if role == QtCore.Qt.BackgroundRole:
            if package.checked:
                return QtCore.QVariant(QtGui.QBrush(QtGui.QColor(QtCore.Qt.green)))
            if package.neededBy:
                return QtCore.QVariant(QtGui.QBrush(QtGui.QColor(QtCore.Qt.yellow)))

        if role == QtCore.Qt.CheckStateRole:
            if index.column()==0:
                return QtCore.QVariant(self.package_checkState(index))

        #if role == QtCore.Qt.DecorationRole:
            #if package.checked:
                #return QtCore.QVariant(checkedPixmap)
            #if package.neededBy:
                #return QtCore.QVariant(neededByPixmap)

        return QtCore.QVariant() # QtCore.QVariant() => None


    def flags(self, index):
        f = QtCore.QAbstractTableModel.flags(self,index)
        if index.column()==0:
            f |= QtCore.Qt.ItemIsUserCheckable
        return f


    #def setData(self,index, data, role):
        ##if index.column()==0:
        ##    self.checks[index.row()]=data.toBool()
        #return True


    def headerData(self, column, orientation, role):
            if orientation == Qt.Horizontal and role == Qt.DisplayRole:
                columnName = self.columns[column]
                columnHeader = self.columnHeaders.get(columnName, columnName)
                return QtCore.QVariant(columnHeader)
            return QtCore.QVariant()


    def checkbox(self,index):
        """
            Check or uncheck the packages checkboxes and their dependencies
            This function is called each time you choose a package (click on
            checkbox)
        """
        if index.column() == 0 :
            package = self.cache[self.list[index.row()]]
            # Clicked, so change the package status :
            package.checked = not package.checked
            if package.checked :
                self.summaryPkgTab.addUpstreamPkg(package.name)
            else:
                self.summaryPkgTab.delUpstreamPkg(package.name)
            # Calculate package dependencies
            deps = set()
            deps = self.dependencies(self.cache,package,deps,"Depends")
            # WTF there is this 4 lines?
            preDeps = set()
            preDeps = self.dependencies(self.cache,package, preDeps, "PreDepends")
            for dep in preDeps:
                deps.add(dep)
            for deppackname in deps:
                deppack = self.cache[deppackname]
                if package.checked:
                    deppack.neededBy.add(package)
                else:
                    deppack.neededBy.remove(package)
                try:
                    row = self.list.index(deppackname)
                    self.emit(QtCore.SIGNAL('dataChanged(QModelIndex,QModelIndex)'),
                        self.index(row,0), self.index(row, len(self.columns)-1))
                except ValueError: pass
            
        self.emit(QtCore.SIGNAL('dataChanged(QModelIndex,QModelIndex)') , index, index )
        # Sort by checked (checkbox column)
            
        return True


    def info(self,pname,infotype):
         #packagesSize
        if infotype == 'pkgsize':
            return self.cache[pname].packageSize
         #Version
        if infotype == 'version':
            return self.cache[pname].candidateVersion


    def sort(self, column, order):
        #print "sort",column,order
        # Don 't sort by summary neither by version
        if column >= 3:
            return
        # Send signal to freeze layout
        self.emit(QtCore.SIGNAL('layoutAboutToBeChanged()'))
        #if column == 1:
        comparator = self.makeComparator(self.columns[column])
        self.list.sort(comparator)
        self.fulllist.sort(comparator)

        # Reverse order
        if order == QtCore.Qt.DescendingOrder:
            self.list.reverse()
            self.fulllist.reverse()
     # Send signal to update layout
        self.emit(QtCore.SIGNAL('layoutChanged()'))


    def makeComparator(self,attrName):
        def comparator(packageName1, packageName2):
            package1 = self.cache[packageName1]
            package2 = self.cache[packageName2]
            if attrName == 'checked':
                result = cmp(len(package1.neededBy), len(package2.neededBy))
            else:
                try: attrValue1 = getattr(package1, attrName)
                except: attrValue1 = None
                try: attrValue2 = getattr(package2, attrName)
                except: attrValue2 = None
                result = cmp(attrValue1, attrValue2)
            return result
        return comparator


    def dependencies(self,cache, pkg, deps, key="Depends"):
        """
            Recursive function which return a set of the package
            dependencies
        """
        #print "pkg: %s (%s)" % (pkg.name, deps)
        # First, check if there is a candidate version for this package
        candver = cache._depcache.GetCandidateVer(pkg._pkg)
        if candver == None:
            return deps
        dependslist = candver.DependsList
        if dependslist.has_key(key):
            for depVerList in dependslist[key]:
                for dep in depVerList:
                    if cache.has_key(dep.TargetPkg.Name):
                       if pkg.name != dep.TargetPkg.Name and not dep.TargetPkg.Name in deps:
                            deps.add(dep.TargetPkg.Name)
                            self.dependencies(cache, cache[dep.TargetPkg.Name], deps, key)
        return deps

    def description(self, index):
        self.Description.setText(self.cache[self.list[index.row()]].description)

    def tsize(self):
        self.tSize.setText( _("Total size: %s") %str(self.formatSize(self.totalSize())))

    def totalSize(self):
        size = sum( [p.packageSize for p in self.cache if p.checked or p.neededBy ] )
        return size

