#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import sys , os, statvfs
from PyQt4 import QtCore, QtGui
from livecreator_misc import functions
from livecreator_misc import config
from livecreator_misc import tips

class PreferencePage(QtGui.QWidget):
    def __init__(self,livecreatorConfig, parent=None):
        QtGui.QWidget.__init__(self, parent)
        # Load application config
        self.livecreatorConfig = livecreatorConfig
        # Create config group box
        configGroup = QtGui.QGroupBox( _("Software settings") )
        # Create main folder group box
        rootGroup = QtGui.QGroupBox( _("Main directory") )
        rootlayout = QtGui.QHBoxLayout()
        self.rootEdit = QtGui.QLineEdit()
        rootBrowser = QtGui.QPushButton( _("Browse") )
        rootlayout.addWidget(self.rootEdit)
        rootlayout.addWidget(rootBrowser)
        rootGroup.setLayout(rootlayout)
        rootGroup.setToolTip(tips.allTips().prefs_root)
        # Create template folder group box
        templateGroup  = QtGui.QGroupBox( _("Templates directory") )
        templatelayout = QtGui.QHBoxLayout()
        self.templateEdit = QtGui.QLineEdit()
        templateBrowser = QtGui.QPushButton( _("Browse") )
        templatelayout.addWidget(self.templateEdit)
        templatelayout.addWidget(templateBrowser)
        templateGroup.setLayout(templatelayout)
        templateGroup.setToolTip(tips.allTips().prefs_template)
        # Create temporary folder group box
        tmpGroup = QtGui.QGroupBox( _("Temporary Directory") )
        tmplayout = QtGui.QGridLayout()
        self.tmpEdit = QtGui.QLineEdit() 
        tmpBrowser = QtGui.QPushButton( _("Browse") )
        self.tmpText = QtGui.QLabel()
        self.tmpWText = QtGui.QLabel()
        self.tmpWText.setText( _("This partition must be in another format than FAT") )
        tmplayout.addWidget(self.tmpWText,0,0,1,6)
        tmplayout.addWidget(self.tmpEdit,1,0,1,5)
        tmplayout.addWidget(tmpBrowser,1,5,1,1)
        tmplayout.addWidget(self.tmpText,2,0,1,6)
        tmpGroup.setLayout(tmplayout)
        tmpGroup.setToolTip(tips.allTips().prefs_tmp)
        # Create image folder group box
        imgGroup = QtGui.QGroupBox( _("Images directory") )
        imglayout = QtGui.QGridLayout()
        self.imgEdit = QtGui.QLineEdit()
        imgBrowser = QtGui.QPushButton( _("Browse") )
        self.imgWText = QtGui.QLabel()
        self.imgWText.setText( _("This partition must be in another format than FAT") )
        imglayout.addWidget(self.imgWText,0,0,1,6)
        imglayout.addWidget(self.imgEdit,1,0,1,5)
        imglayout.addWidget(imgBrowser,1,5,1,1)
        imglayout.addWidget(QtGui.QLabel(" \n "),2,0,1,6)
        imgGroup.setLayout(imglayout)
        imgGroup.setToolTip(tips.allTips().prefs_image)
        # Create hook scripts folder group box
        hookGroup = QtGui.QGroupBox( _("Personaliation scripts directory") )
        hooklayout = QtGui.QHBoxLayout()
        self.hookEdit = QtGui.QLineEdit()
        hookBrowser = QtGui.QPushButton( _("Browse") )
        hooklayout.addWidget(self.hookEdit)
        hooklayout.addWidget(hookBrowser)
        hookGroup.setLayout(hooklayout)
        hookGroup.setToolTip(tips.allTips().prefs_hook)
        # Create local packages folder group box
        pkgGroup = QtGui.QGroupBox( _("Local packages directory") )
        pkglayout = QtGui.QHBoxLayout()
        self.pkgEdit = QtGui.QLineEdit()
        pkgBrowser = QtGui.QPushButton( _("Browse") )
        pkglayout.addWidget(self.pkgEdit)
        pkglayout.addWidget(pkgBrowser)
        pkgGroup.setLayout(pkglayout)
        pkgGroup.setToolTip(tips.allTips().prefs_local)
        # Create save preferences button 
        self.saveConfigButton = QtGui.QPushButton( _("Save settings") )
        self.saveConfigButton.setIcon(QtGui.QIcon("images/save.png"))
        # Create undo preferences button
        self.initConfigButton = QtGui.QPushButton( _("Restore previous settings") )
        self.initConfigButton.setIcon(QtGui.QIcon("images/undo.png"))
        # Create mirror group box
        depotGroup = QtGui.QGroupBox( _("Main repository") )
        depotlayout = QtGui.QHBoxLayout()
        self.depotEdit = QtGui.QLineEdit()
        depotlayout.addWidget(self.depotEdit)
        depotGroup.setLayout(depotlayout)
        depotGroup.setToolTip(tips.allTips().prefs_dep)
        # Create security mirror group box
        depotsecurityGroup = QtGui.QGroupBox( _("Security repository") )
        depotsecuritylayout = QtGui.QHBoxLayout()
        self.depotsecurityEdit = QtGui.QLineEdit()
        depotsecuritylayout.addWidget(self.depotsecurityEdit)
        depotsecurityGroup.setLayout(depotsecuritylayout)
        depotsecurityGroup.setToolTip(tips.allTips().prefs_depsecu)
        # Create ftp proxy group box
        proxyftpGroup = QtGui.QGroupBox( _("FTP proxy") )
        proxyftplayout = QtGui.QHBoxLayout()
        self.proxyftpEdit = QtGui.QLineEdit()
        proxyftplayout.addWidget(self.proxyftpEdit)
        proxyftpGroup.setLayout(proxyftplayout)
        proxyftpGroup.setToolTip(tips.allTips().prefs_ftpproxy)
        # Create http proxy group box
        proxyhttpGroup = QtGui.QGroupBox( _("HTTP proxy") )
        proxyhttplayout = QtGui.QHBoxLayout()
        self.proxyhttpEdit = QtGui.QLineEdit()
        proxyhttplayout.addWidget(self.proxyhttpEdit)
        proxyhttpGroup.setLayout(proxyhttplayout)
        proxyhttpGroup.setToolTip(tips.allTips().prefs_httpproxy)
        # Fill all preferences
        self.init()
        self.checkFreeSpace(self.tmpEdit.text())
        # Show all group boxes
        configLayout = QtGui.QGridLayout()
        configLayout.addWidget(rootGroup,0,0,1,1)
        configLayout.addWidget(templateGroup,0,1,1,1)
        configLayout.addWidget(tmpGroup,1,0,1,1)
        configLayout.addWidget(imgGroup,1,1,1,1)
        configLayout.addWidget(hookGroup,2,1,1,1)
        configLayout.addWidget(pkgGroup,2,0,1,1)
        configLayout.addWidget(depotGroup,3,0,1,1)
        configLayout.addWidget(depotsecurityGroup,3,1,1,1)
        configLayout.addWidget(proxyftpGroup,4,0,1,1)
        configLayout.addWidget(proxyhttpGroup,4,1,1,1)
        configLayout.addWidget(self.saveConfigButton,5,1)
        configLayout.addWidget(self.initConfigButton,5,0)
        configGroup.setLayout(configLayout)
        mainLayout = QtGui.QGridLayout()
        mainLayout.addWidget(configGroup,0,0,0,12)
        self.setLayout(mainLayout)
        # Signals
        QtCore.QObject.connect(self.initConfigButton ,QtCore.SIGNAL('clicked()'),self.reinit )
        QtCore.QObject.connect(self.saveConfigButton ,QtCore.SIGNAL('clicked()'),self.saveconfig )
        QtCore.QObject.connect(rootBrowser ,QtCore.SIGNAL('clicked()'),self.rootBrowse )
        QtCore.QObject.connect(templateBrowser ,QtCore.SIGNAL('clicked()'),self.templateBrowse )
        QtCore.QObject.connect(tmpBrowser ,QtCore.SIGNAL('clicked()'),self.tmpBrowse )
        QtCore.QObject.connect(imgBrowser ,QtCore.SIGNAL('clicked()'),self.imgBrowse )
        QtCore.QObject.connect(hookBrowser ,QtCore.SIGNAL('clicked()'),self.hookBrowse )
        QtCore.QObject.connect(pkgBrowser ,QtCore.SIGNAL('clicked()'),self.pkgBrowse )
        QtCore.QObject.connect(self.tmpEdit ,QtCore.SIGNAL('textEdited(QString)'),self.checkFreeSpace )
        

    def saveconfig(self):
        """
            Collect preference values and send its to
            config.writeOptionValues function to write
            the new preference file.
        """
        optionsList = {'rootdir':self.rootEdit.text(),
            'templatedir':self.templateEdit.text(),
            'tmpdir':self.tmpEdit.text(),
            'imgdir':self.imgEdit.text(),
            'hookdir':self.hookEdit.text(),
            'pkgdir':self.pkgEdit.text(),
            'depurl':self.depotEdit.text(),
            'secudepurl':self.depotsecurityEdit.text(),
            'ftpproxy':self.proxyftpEdit.text(),
            'httpproxy':self.proxyhttpEdit.text()}
        for value in optionsList:
            if optionsList[value] == "" and value != "ftpproxy" and value != "httpproxy" :
                question = QtGui.QMessageBox( _("Information"),
                    _("Some fields are blank.<br>Settings will not be saved"),
                    QtGui.QMessageBox.Information, 
                    QtGui.QMessageBox.Yes, 
                    0, 
                    0 )
                question.setButtonText( QtGui.QMessageBox.Yes, _("Ok") )
                question.exec_()
                return
        # Options are OK
        question = QtGui.QMessageBox( _("Question"),
            _("Save settings?"),
                QtGui.QMessageBox.Question, 
                QtGui.QMessageBox.Yes, 
                QtGui.QMessageBox.No, 
                0 )
        question.setButtonText( QtGui.QMessageBox.Yes, _("Yes") )
        question.setButtonText( QtGui.QMessageBox.No, _("No") )
        if question.exec_() == QtGui.QMessageBox.Yes:
            config.writeOptionValues(self.livecreatorConfig, optionsList)

    def reinit(self):
        """
            Reinitiliaze previous preferences
        """
        question = QtGui.QMessageBox( _("Question"),
            _("Reinitialise settings?"),
                QtGui.QMessageBox.Question, 
                QtGui.QMessageBox.Yes, 
                QtGui.QMessageBox.No, 
                0 )
        question.setButtonText(QtGui.QMessageBox.Yes, _("Yes") )
        question.setButtonText(QtGui.QMessageBox.No,_("No") )
        if question.exec_() == QtGui.QMessageBox.Yes:
            self.init()


    def init(self):
        """
            Read preferences file
        """
        self.rootEdit.setText(self.livecreatorConfig.get("main","rootdir"))
        self.tmpEdit.setText(self.livecreatorConfig.get("main","tmpdir"))
        self.imgEdit.setText(self.livecreatorConfig.get("main","imgdir"))
        self.hookEdit.setText(self.livecreatorConfig.get("main","hookdir"))
        self.pkgEdit.setText(self.livecreatorConfig.get("main","pkgdir"))
        self.templateEdit.setText(self.livecreatorConfig.get("main","templatedir"))
        self.depotsecurityEdit.setText(self.livecreatorConfig.get("main","secudepurl"))
        self.depotEdit.setText(self.livecreatorConfig.get("main","depurl"))
        self.proxyftpEdit.setText(self.livecreatorConfig.get("main","ftpproxy"))
        self.proxyhttpEdit.setText(self.livecreatorConfig.get("main","httpproxy"))

    def rootBrowse(self):
        """
            Browse to select the root folder
        """
        file = QtGui.QFileDialog.getExistingDirectory (self, _("Main directory"),
                                                           QtCore.QDir.currentPath())
        if file:
            self.rootEdit.setText(file)


    def templateBrowse(self):
        """
            Browse to select the template folder
        """
        file = QtGui.QFileDialog.getExistingDirectory (self, _("Templates directory"),
                                                           QtCore.QDir.currentPath())
        if file:
            self.templateEdit.setText(file)


    def tmpBrowse(self):
        """
            Browse to select the tmp folder
        """
        file = QtGui.QFileDialog.getExistingDirectory (self, _("Temporary directory"),
                                                           QtCore.QDir.currentPath())
        if file:
            self.tmpEdit.setText(file)
            self.checkFreeSpace(self.tmpEdit.text())


    def imgBrowse(self):
        """
            Browse to select the image folder
        """
        file = QtGui.QFileDialog.getExistingDirectory (self, _("Images directory"),
                                                           QtCore.QDir.currentPath())
        if file:
            self.imgEdit.setText(file)


    def hookBrowse(self):
        """
            Browse to select the hook script folder
        """
        file = QtGui.QFileDialog.getExistingDirectory (self, _("Personalization scripts directory"),
                                                           QtCore.QDir.currentPath())
        if file:
            self.hookEdit.setText(file)


    def pkgBrowse(self):
        """
            Browse to select the packages folder
        """
        file = QtGui.QFileDialog.getExistingDirectory (self, _("Local packages directory"),
                                                           QtCore.QDir.currentPath())
        if file:
            self.pkgEdit.setText(file)


    def existFolder(self,folder):
        """
            Check if folder exist or not
            If yes, return the descriptor
        """
        try :
            s = os.statvfs(folder)
        except Exception, e:
            self.tmpText.setText( _('''<font color="red">Directory doesn't exist.</font><br>''') )
            return False
        return s


    def checkFreeSpace(self,folder):
        """
            Check free space on a hdd
        """
        folder = str(folder)
        s = self.existFolder(folder)
        if s != False:
            # Get freespace
            freebytes = s[statvfs.F_BSIZE] * s[statvfs.F_BAVAIL]
            # if freespace 
            if freebytes < 15*1024*1024*1024:
                # Format free space
                freeformat = functions.format_size(freebytes)
                self.tmpText.setText( _('''<font color="red">Directory must provide at least 15G..<br/>Just %s are available.</font>''') %freeformat)
            else:
                self.tmpText.setText( _("Directory must provide at least 15G.") )





