#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys
from PyQt4 import QtCore, QtGui
from livecreator_misc import functions
import os
#import subprocess as sp
import time
from livecreator_misc import config

class genImageProgress(QtGui.QDialog):
    def __init__(self, genprocess, parent=None):
        """
            First dialog box. It permit to choose
            which device will be use to make the live
        """
        QtGui.QDialog.__init__(self, parent)

        # Step define the current step
        self.step = 0

        # Generation process
        self.genprocess = genprocess

        self.setFixedSize(300, 150)
        self.setWindowModality(QtCore.Qt.WindowModal)
        self.setWindowTitle( _("Live generation processus") )

        self.stepLabel = QtGui.QLabel( _("Generation") )
        self.stepLabel.setAlignment(QtCore.Qt.AlignHCenter)
        self.stepProgress = QtGui.QProgressBar()
        self.stepProgress.setTextVisible(0)
        self.stepProgress.setMaximum(0)
        self.stepProgress.setMinimum(0)

        self.progressLabel = QtGui.QLabel( _("Global process") )
        self.progressLabel.setAlignment(QtCore.Qt.AlignHCenter)
        self.totalProgress = QtGui.QProgressBar()

        self.cancelButton = QtGui.QPushButton( _("Cancel generation") )

        mainLayout = QtGui.QGridLayout()
        mainLayout.addWidget(self.stepLabel,0,0,1,1)
        mainLayout.addWidget(self.stepProgress,1,0,1,1)
        mainLayout.addWidget(self.progressLabel,2,0,1,1)
        mainLayout.addWidget(self.totalProgress,3,0,1,1)
        mainLayout.addWidget(self.cancelButton,4,0,1,1)

        QtCore.QObject.connect(self.cancelButton ,QtCore.SIGNAL('clicked()'),self.cancel )

        self.setLayout(mainLayout)

        self.timer = QtCore.QTimer(self)
        self.timer.start(1000)
        self.connect(self.timer, QtCore.SIGNAL("timeout()"),self.setStepProgressBar)


    def setStepProgressBar(self):
        """
            Set step progress
        """
        self.timer.stop()
        self.stepProgress.setValue(self.stepProgress.value()+5)
        if self.stepProgress.value() >= 96:
            self.stepProgress.setValue(0)
        self.timer.start(1000)



    def processing(self,output):
        """
            Change display and progress bars in function of the output
            TODO : change this comment in English (keep sames sentences than in method)
            Steps:"I: " : "Préparation de l'environnement",
                  "P: Configuring file /etc/kernel-img.conf" : "Installation de l'environnement",
                  "P: Begin installing packages..." : "Installation des paguets",
                  "[" : "Création de l'image",
                  "memtest86+" : "vérification de l'image"
                  "memtest86+" : "Finalisation de l'image"
        """
        # Set step Progress Bar to 0% if it is at 100%
        
        if self.stepProgress.value() >= 96 and self.step != 4:
            self.stepProgress.setValue(0)
        # Set display and progress bars
        # Step 1 :
        if output.startswith("I: ") and self.step == 0 :
            self.step = 1
            self.totalProgress.setValue(15)
            self.stepLabel.setText( _("Environnement preparation") )
            self.stepProgress.setTextVisible(0)
        # Step 2 :
        if output.startswith("P: Configuring file /etc/kernel-img.conf") and self.step == 1:
            self.step = 2
            self.totalProgress.setValue(40)
            self.stepLabel.setText( _("Environement installation") )
            self.stepProgress.setTextVisible(0)
        # Step 3 :
        if output.startswith("P: Begin installing packages...") and self.step == 2:
            self.step = 3
            self.totalProgress.setValue(50)
            self.stepLabel.setText( _("Packages installation...") )
            self.stepProgress.setTextVisible(0)
        # Step 4 :
        if output.endswith("%") and self.step == 3:
            self.step = 4
            self.totalProgress.setValue(65)
            self.stepLabel.setText( _("Image creation") )
            self.stepProgress.setTextVisible(1)
            # Stop timer for step progress bar
            self.timer.stop()
            self.stepProgress.setMaximum(100)
            self.stepProgress.setMinimum(0)
            self.stepProgress.setValue(0)
        # Crypt
        #if output.startswith("Encrypting binary/live/filesystem.squashfs with") or output.startswith("Password:"):
            #passDialog = genPasswordDialog(self.genprocess)
            #passbox = -1
            #while passbox == -1:
                #passbox = passDialog.exec_()
        # Step 5:
        if output.find("Reading package lists") != -1 and self.step == 4:
            self.step = 5
            self.totalProgress.setValue(80)
            self.stepLabel.setText( _("Checking image") )
            self.stepProgress.setTextVisible(0)
            self.stepProgress.setMaximum(0)
            self.stepProgress.setMinimum(0)
            # Stop timer for step progress bar
            self.timer.start(1000)
        # Step 6 :
        if output.find("memtest86+") != -1 and self.step == 5:
            self.step = 6
            self.totalProgress.setValue(90)
            self.stepLabel.setText( _("Image finalisation") )
            self.stepProgress.setTextVisible(0)


        # Set Value of progress for step 1, 2, 3, 5 and 6.
        if output.endswith("%") and self.step == 4:
            self.stepLabel.setText( _("Image creation") )
            self.stepProgress.setValue(int(output[-3:-1]))
       # else:
           # self.stepProgress.setValue(self.stepProgress.value()+5)


    def cancel(self):
        """
            Launch check box and close the box
        """
        genCancel = genCancelDialog()
        if genCancel.exec_() == QtGui.QMessageBox.Yes:
            # Set generation process error to 1
            self.genprocess.error = 1
            # Stop generation
            self.genprocess.close()
            # Stop timer for step progress bar
            self.timer.stop()
            # Close the windows
            self.done(-1)


    def closeDiag(self):
        """
            Launch check box and close the box
        """
        # Stop generation
        self.genprocess.close()
        # Stop timer for step progress bar
        self.timer.stop()
        # Close the windows
        self.done(-1)


    def closeEvent(self, event):
        """
            Don't close progress bar when
            user click on the X (close)
        """
        event.ignore()


class genCancelDialog(QtGui.QMessageBox):
    def __init__(self, parent=None):
        """
            Warning message to be sure that
            user want to stop generation
        """
        QtGui.QMessageBox.__init__(self, _("Cancel image generation"),
                                    "",
                                    QtGui.QMessageBox.Warning,
                                    QtGui.QMessageBox.Yes,
                                    QtGui.QMessageBox.No,
                                    0)
        self.setButtonText(QtGui.QMessageBox.Yes, _("&Yes") )
        self.setButtonText(QtGui.QMessageBox.No,_("&No") )
        self.setText( _('<font color="red"><b>Do you really want cancel generation?</b></font>') )


class genCleanDialog(QtGui.QDialog):
    def __init__(self, livecreatorConfig, parent=None):
        """
            >arning message to be sure that
            user want to stop generation
        """
        self.livecreatorConfig = livecreatorConfig
        QtGui.QDialog.__init__(self,parent)

        self.setFixedSize(200, 100)

        self.setWindowTitle( _("Cleanning...") )

        self.stepLabel = QtGui.QLabel( _("Cleaning...<br>Please wait.") )
        self.stepLabel.setAlignment(QtCore.Qt.AlignHCenter)

        self.timer = QtCore.QTimer(self)

        mainLayout = QtGui.QGridLayout()
        mainLayout.addWidget(self.stepLabel,0,0,1,1)
        self.setLayout(mainLayout)

        self.timer.start(1000)
        self.connect(self.timer, QtCore.SIGNAL("timeout()"),self.clean)


    def clean(self):
        config.cleanWorkSpace(self.livecreatorConfig)
        self.timer.stop()
        self.done(-1)


    def closeEvent(self, event):
        """
            Don"t close progress bar when
            user click on the X (close)
        """
        event.ignore()


class genPasswordDialog(QtGui.QDialog):
    def __init__(self, process, parent=None):
        """
            Dialogbox for ask a password
        """
        QtGui.QDialog.__init__(self,parent)

        self.process = process

        self.setWindowTitle( _("Password") )

        self.stepLabel1 = QtGui.QLabel( _("Please enter a new passphrase") )
        self.stepLabel1.setAlignment(QtCore.Qt.AlignHCenter)
        self.password1 = QtGui.QLineEdit()
        self.stepLabel2 = QtGui.QLabel( _("Please renter same passphrase") )
        self.stepLabel2.setAlignment(QtCore.Qt.AlignHCenter)
        self.password2 = QtGui.QLineEdit()
        self.passwordButton = QtGui.QPushButton( _("OK") )


        mainLayout = QtGui.QGridLayout()
        mainLayout.addWidget(self.stepLabel1,0,0,1,1)
        mainLayout.addWidget(self.password1,1,0,1,1)
        mainLayout.addWidget(self.stepLabel2,2,0,1,1)
        mainLayout.addWidget(self.password2,3,0,1,1)
        mainLayout.addWidget(self.passwordButton,4,0,1,1)
        self.setLayout(mainLayout)

        QtCore.QObject.connect(self.passwordButton ,QtCore.SIGNAL('clicked()'),self.sendPass )

    def sendPass(self):
        """
            dialog box to check password 
            and send it to the generation process
        """
        # Password
        pass1 = self.password1.text()
        pass2 = self.password2.text()
        # Check if passwords match
        #print pass2
        #print pass1
        if str(pass1) == str(pass2):
            # Check if password contain 20 caracters
            if pass1.count() == 20:
                # Send password to the generation process
                self.process.writeData("0123456789\n" )
                self.process.writeData("0123456789\n" )
                #procxterm.writeData(str(pass1)+"\n" )
                #print str(pass1)
                #procxterm.writeData(str(pass1)+"\n" )
                #print str(pass1)
                #self.process.write(str(pass1) )
                #self.process.waitForBytesWritten(-1)
                #self.process.closeWriteChannel()
                #self.process.write(str(pass1) )
                #self.process.waitForBytesWritten(-1)
                #self.process.closeWriteChannel()
                self.done(1)
            else:
                passError = QtGui.QMessageBox( _("Passphrase error"),
                            _("Passphrase must contain 20 characters"),
                            QtGui.QMessageBox.Critical,
                            QtGui.QMessageBox.Yes,
                            0,
                            0)
                passError.setButtonText(QtGui.QMessageBox.Yes, _("&Ok") )
                passError.exec_()
                print _("Passphrase must contain 20 characters")
                self.done(-1)
        else:
            passError = QtGui.QMessageBox( _("Passphrase error"),
                        _("The two passphrases are different"),
                        QtGui.QMessageBox.Critical,
                        QtGui.QMessageBox.Yes,
                        0,
                        0)
            passError.setButtonText(QtGui.QMessageBox.Yes, _("&Ok") )
            passError.exec_()
            print _("The two passphrases are different"),
            self.done(-1)


    def closeEvent(self, event):
        """
            Don"t close progress bar when
            user click on the X (close)
        """
        event.ignore()

