#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys, os, shutil
from PyQt4 import QtCore, QtGui
from livecreator_misc import config
from time import localtime, strftime
import genimagedialog as genimage
from livecreator_misc import tips

class SummaryPage(QtGui.QWidget):
    def __init__(self,livecreatorConfig,xmlFile, testImgPage, parent=None):
        QtGui.QWidget.__init__(self, parent)

        self.testImgPage = testImgPage
        self.allOptionsList = []
        self.livecreatorConfig = livecreatorConfig
        self.xmlFile = xmlFile

        summaryGroup = QtGui.QGroupBox( _("Detailled template summary") )

        self.summaryText = QtGui.QTextEdit()

        self.summaryText.setReadOnly(1)

        summaryLayout = QtGui.QHBoxLayout()

        # XTERM handler defintion
        # We get the summaryLayout window id to attach xterm into it
        self.opt_list = ['-l  sleep 15'] #toexecute]
        self.procxterm = QtCore.QProcess()
        self.procxterm.error = 0

        summaryGroup.setLayout(summaryLayout)
        summaryLayout.addWidget(self.summaryText)

        self.GenImage = QtGui.QPushButton( _("Live image generation") )
        self.GenImage.setToolTip(tips.allTips().summary_gen)

        mainLayout = QtGui.QVBoxLayout()
        mainLayout.addWidget(summaryGroup)
        mainLayout.addWidget(self.GenImage)

        QtCore.QObject.connect(self.GenImage ,QtCore.SIGNAL('clicked()'),self.generateImage )
        self.connect(self.procxterm, QtCore.SIGNAL('readyReadStandardOutput()'), self.readOutput)
        self.connect(self.procxterm, QtCore.SIGNAL('readyReadStandardError()'), self.readErrorOutput)
        self.connect(self.procxterm, QtCore.SIGNAL('stateChanged (QProcess::ProcessState)'), self.state)

        self.setLayout(mainLayout)


    def readErrorOutput(self):
        """
            Read the process standards Outputs
        """
        # Place the cursor to the end of the textedit
        self.summaryText.moveCursor(QtGui.QTextCursor.End)
        # Write output
        self.summaryText.insertPlainText(QtCore.QString(self.procxterm.readAllStandardError()))


    def readOutput(self):
        """
            Read the process standards Outputs
        """
        output = str(self.procxterm.readAllStandardOutput())
        self.imageProgressdialog.processing(output)
        # Place the cursor to the end of the textedit
        self.summaryText.moveCursor(QtGui.QTextCursor.End)
        # Write output
        if output.find("%") == -1 :
            self.summaryText.insertPlainText(QtCore.QString(output))


    def generateImage(self):
        """
            Launch the live_helper backend to build the image.
        """
        # The options list passed to lh_config
        # This list will be modified in this method
        opt_list = ""
        # The option store the "very advanced option
        # line" in advanced options
        opt_supp = ""
        # A String to store the different options (packages list, username, ...)
        optionValue = ""
        # String for all bootappend options
        bootappend = ""
        # String for all operations between config and build
        before_build_operations = ""
        # String for all KERNEL operations between config and build
        changekernel = ''
        # String for all KERNEL append 
        append = ''
        # Variable to remember and to know
        # if interactive mode is activated
        interMode = 0

        # Open a confirmation  box before launch process
        question =  QtGui.QMessageBox( _("Question"),
                    _("Generate Live image?"),
                    QtGui.QMessageBox.Question, 
                    QtGui.QMessageBox.Yes, 
                    QtGui.QMessageBox.No, 
                    0 )
        question.setButtonText(QtGui.QMessageBox.Yes, _("Yes"))
        question.setButtonText(QtGui.QMessageBox.No, _("No"))
        if question.exec_() == QtGui.QMessageBox.Yes:
            # Descativate the button during the process
            self.GenImage.setDisabled(1)
            # Activate all the general options
            # The structure of self.allOptionsList is :
            # Typical option : ("Name", "Value", "XmlName", "LiveHelperOption / LiveCreator Option")
            # example :        ("Nom de l'utilisateur", "user1", "username", "--username")
            for element_tuple in self.allOptionsList:
                if len(element_tuple) > 3:
                    livecreatorOption = element_tuple[3]
                    value = element_tuple[1]
                    if livecreatorOption.startswith("--"):
                        # element_tuple[1] is a PyQt4.QtCore.QString object
                        if type(value) is type(QtCore.QString()):
                            optionValue = str(value)
                        # or a PyQt4.QtGui.QListWidget object
                        if type(value) is type(QtGui.QListWidget()) :
                            # translate QListWidget to python list
                            for index in range(value.count()):
                                optionValue += " " + str(value.item(index).text())
                        # We doesn't want add an empty option
                        if optionValue != "":
                            # Test if livecreatorOption if --package to know 
                            # if this option is already set by the interactive mode
                            if livecreatorOption == "--packages":
                                #Test if --package is already set or not
                                if str(opt_list).find("--packages") != -1:
                                    # If yes add dialog packages list
                                    opt_list = str(opt_list).replace('--packages "', '--packages " ' + optionValue + " ")
                                else:
                                    # If no add "--packages" option and dialog
                                    opt_list += ''' --packages "''' + optionValue + '"'
                            # if livecreatorOption is NOT --packages
                            else:
                                opt_list +=" " + livecreatorOption + ' "' + optionValue + '"'
                        optionValue = ""
                    # If the livecreatorOption don't starts with "--", then it's a particular case
                    else:
                        # Define the keymap
                        if livecreatorOption == "BOOTAPPEND_KBD":
                            if str(value) != _("Choose during Live boot"):
                                # For keyboard layout we made a mask in the combox (customizepage => gencustomtab => kbdCombo )
                                # We show language (layout) and the data is the layout which is
                                # store at the end of the tuple
                                # So we need to take the last (4ft) item of the element_tuple
                                value = str(element_tuple[4])
                                opt_list +=" -l " + value[0:2] + " "
                                bootappend += " keyb=" + value + " "
                                # Copy chosen keyboard in /tmp for encryption
                                # Read keymaps folder and subfolders
                                destKeyboard  ="/tmp/boot_kmap.gz"
                                qKeymapDir = QtCore.QDir("/usr/share/keymaps/")
                                if qKeymapDir.exists():
                                    keymapsDir = QtCore.QDirIterator(qKeymapDir,QtCore.QDirIterator.Subdirectories)
                                    while keymapsDir.hasNext():
                                        # Read file name
                                        keymap = str(keymapsDir.filePath())
                                        # If the keymap is the chosen keymap
                                        # We copy it in tmp folder
                                        if keymapsDir.fileName() == value+".kmap.gz":
                                            before_build_operations += " && cp " + keymap + " " + destKeyboard
                                            break
                                        keymapsDir.next()
                                #opt_list +=" -l " + str(value) + " "
                                #bootappend += self.setkbd(str(value))
                            else:
                                # Interactive mode
                                # Test if interactive mode is already activated or not
                                (interMode, bootappend, before_build_operations, opt_list) = self.interactiveMode(interMode, bootappend, before_build_operations, opt_list)
                                # Make stuff to keyboard interactive mode
                                destDir = "./config/chroot_local-includes/etc/init.d/"
                                file = os.getcwd() + "/specific_hooks/ask_kbd.sh"
                                before_build_operations += " && mkdir -p " + destDir + " && cp " + file + " " + destDir
                                before_build_operations +=  " && cd ./config/chroot_local-includes/"
                                before_build_operations += " && ln -s "+ "/etc/init.d/ask_kbd.sh " +  "./etc/rc4.d/S00ask_kbd.sh"
                                before_build_operations += "&& cd ../../"
                        # Change the locales
                        elif livecreatorOption == "LANG":
                            if str(value) != _("Choose during Live boot"):
                                # For language we made a mask in the combox (customizepage => gencustomtab => langCombo )
                                # We show country (language) and the data is the locale of the country which is
                                # store at the end of the tuple
                                # So we need to take the last (4ft) item of the element_tuple
                                value = element_tuple[4]
                                bootappend += self.setlang(str(value))
                            else:
                                # Interactive mode
                                # Test if interactive mode is already activated or not
                                (interMode, bootappend, before_build_operations, opt_list) = self.interactiveMode(interMode, bootappend, before_build_operations, opt_list)
                                # Make stuff to language interactive mode
                                destDir = "./config/chroot_local-includes/etc/init.d/"
                                file = os.getcwd() + "/specific_hooks/ask_lang.sh"
                                before_build_operations += " && mkdir -p " + destDir + " && cp " + file + " " + destDir
                                before_build_operations +=  " && cd ./config/chroot_local-includes/"
                                before_build_operations += " && ln -s "+ "/etc/init.d/ask_lang.sh " +  "./etc/rc4.d/S00ask_lang.sh "
                                before_build_operations += "&& cd ../../"
                        # Manage DHCP
                        elif livecreatorOption == "DHCP":
                            switch = str(value)
                            if switch != _("Choose during Live boot"):
                                # Desactivate DHCP on boot
                                if switch == "Off":
                                    # Force liveinitramfs to take netwok configuration from files
                                    bootappend += " ip=frommedia "
                                    # We need to copy some files
                                    destDir = "./config/chroot_local-includes/etc/network"
                                    file = os.getcwd() + "/specific_hooks/disable-services_overwrite/etc/network/interfaces"
                                    before_build_operations += " && mkdir -p " + destDir + " && cp " + file + " " + destDir
                            else:
                                # Interactive mode
                                # Test if interactive mode is already activated or not
                                (interMode, bootappend, before_build_operations, opt_list) = self.interactiveMode(interMode, bootappend, before_build_operations, opt_list)
                                # Make stuff to DHCP interactive mode
                                destDir = "./config/chroot_local-includes/etc/init.d/"
                                file = os.getcwd() + "/specific_hooks/ask_network.sh"
                                before_build_operations += " && mkdir -p " + destDir + " && cp " + file + " " + destDir
                                before_build_operations +=  " && cd ./config/chroot_local-includes/"
                                before_build_operations += " && ln -s "+ "/etc/init.d/ask_network.sh " +  "./etc/rc4.d/S00ask_network.sh"
                                before_build_operations += "&& cd ../../"
                        # Add local packages copying them into the specific livehelper directory
                        elif livecreatorOption == "LOCALDEBS":
                            if value.count() > 0:
                                before_build_operations += " && cp"
                                # translate QListWidget to python list
                                for index in range(value.count()):
                                    # local Package
                                    localpkg = str(value.item(index).text())
                                    # Copying all local package in chroot_local-packages folder
                                    # before lh_build
                                    before_build_operations += " " + self.livecreatorConfig.get("main","pkgdir") + "/" + localpkg 
                                    # Test if the local package is a kernel-linux
                                    if localpkg.startswith("linux-image-"):
                                        # If Yes, We have to do some change in
                                        # config/chroot
                                        changekernel = 1
                                        # find kernel append
                                        if localpkg and localpkg.split("_")[0] and localpkg.split("_")[0].split("-")[-1]:
                                            append += localpkg.split("_")[0].split("-")[-1] + " "
                                before_build_operations += " ./config/chroot_local-packages"
                                # changekernel is equal to 1 if at least one kernel
                                # was detected. So we have to make the script# to change
                                # config/chroot
                                if changekernel == 1:
                                    changekernel = '''sed -i 's/^LH_LINUX_PACKAGES=.*/LH_LINUX_PACKAGES="none"/g' ./config/chroot  && sed -i 's/^LH_LINUX_FLAVOURS=.*/LH_LINUX_FLAVOURS=" ''' + append + """  "/g' ./config/chroot """
                        # Activate the automated sources compilations
                        elif livecreatorOption == "SOURCES_PACKAGES":
                            if value.count() > 0:
                                # We copy the hook script
                                file = os.getcwd() + "/specific_hooks/"
                                destDir = "./config/chroot_local-hooks/"
                                before_build_operations += " && cp " + file + "compile_all_classic_way.sh " + destDir
                                # Copying all tar.gz files
                                destDir = "./config/chroot_local-includes/root/packages"
                                before_build_operations += " && mkdir -p " + destDir + " && cp"
                                for index in range(value.count()):
                                    before_build_operations += " " +  self.livecreatorConfig.get("main","pkgdir") + "/" + str(value.item(index).text())
                                before_build_operations += " " + destDir
                        # Handle the disks automount
                        elif livecreatorOption == "AUTOMOUNT":
                            if str(value) != _("Choose during Live boot"):
                                scripts = { "On": "mountall.sh ", "Forensic": "mountall_forensic.sh "}
                                if str(value) != "Off":
                                    file = os.getcwd() + "/specific_hooks/disable-services_overwrite/etc/init.d/"
                                    destDir = "./config/chroot_local-includes/etc/init.d/"
                                    before_build_operations += " && mkdir -p " + destDir + " && cp " + file + scripts[str(value)] + destDir
                                    file = os.getcwd() + "/specific_hooks/disable-services_overwrite/etc/rc2.d/"
                                    destDir = "./config/chroot_local-includes/etc/rc2.d/"
                                    before_build_operations += " && mkdir -p " + destDir + " && cp " + file + "S99" + scripts[str(value)] + destDir
                            else:
                                # Interactive mode
                                # Test if interactive mode is already activated or not
                                (interMode, bootappend, before_build_operations, opt_list) = self.interactiveMode(interMode, bootappend, before_build_operations, opt_list)
                                # Make stuff to automount interactive mode
                                destDir = "./config/chroot_local-includes/etc/init.d/"
                                file = os.getcwd() + "/specific_hooks/ask_mount.sh"
                                before_build_operations += " && mkdir -p " + destDir + " && cp " + file + " " + destDir
                                before_build_operations +=  " && cd ./config/chroot_local-includes/"
                                before_build_operations += " && ln -s "+ "/etc/init.d/ask_mount.sh " +  "./etc/rc4.d/S00ask_mount.sh"
                                before_build_operations += "&& cd ../../"
                        # Activate the interactive mode or not
                        #elif livecreatorOption == "INTERACTIVE_MODE":
                            #if value == "On":
                                ## We boot in runlevel 4
                                #bootappend += " 4 "
                                ## We need to copy some files
                                #file = os.getcwd() + "/specific_hooks/"
                                #destDir = "./config/chroot_local-includes/"
                                #before_build_operations += " && rsync -a " + file + "disable-services_overwrite/etc " + destDir
                                #before_build_operations += " && cp " + file + "disable-run4.sh ./config/chroot_local-hooks"
                        # Add encryption. We have to do some operation since live helper don't work very well on it
                        elif livecreatorOption == "ENCRYPTION":
                            if value != "Off":
                                opt_list += ' --losetup "losetup" --encryption ' + value
                                # Add the hooks scripts
                                file = os.getcwd() + "/specific_hooks/"
                                destDir = "./config/chroot_local-hooks"
                                # A hook to install dependences
                                before_build_operations += " && cp " + file + "install_crypto_stuff_chroot.sh " + destDir
                                # Another hook to correct the crypto
                                destDir = "./config/binary_local-hooks"
                                before_build_operations += " && cp " + file + "install_crypto_stuff.sh " + destDir
                        # Add Optional lh_config options
                        elif livecreatorOption == "SUPP":
                            opt_supp = " " + value + " "
                        # Add the usplash
                        elif livecreatorOption == "USPLASH":
                            if len(str(value)) > 0:
                                file = os.getcwd() + "/specific_hooks/"
                                # Prepare environment
                                # Create a QT Image to be able to convert it into right format
                                tempSplash = QtGui.QImage(str(value))
                                tempSplash = tempSplash.convertToFormat (QtGui.QImage.Format_Indexed8)
                                tempSplash = tempSplash.scaled(640,480)
                                tempSplashFile = "/tmp/splash.png"
                                tempSplash.save(tempSplashFile, "PNG")
                                # We now have the good format. Let's put it in right place
                                destDir = "./config/chroot_local-includes/"
                                before_build_operations += " && mkdir -p " + destDir + "/tmp" " && cp " + file + "usplash-theme-ubuntu_0.17.tar.gz " + destDir + "/tmp"
                                before_build_operations += " && cp " + tempSplashFile + " " + destDir + "/tmp"
                                destDir = "./config/chroot_local-hooks/"
                                before_build_operations += " && cp " + file + "01create_usplash_theme.sh " + destDir
                                before_build_operations += " && cp " + file + "02usplash_theme.sh " + destDir
                        # Add hooks (binary or chroot)
                        elif livecreatorOption == "HOOKS_BINARY" or livecreatorOption == "HOOKS_CHROOT":
                            destDir = { "HOOKS_BINARY": "./config/binary_local-hooks", "HOOKS_CHROOT": "./config/chroot_local-hooks" }
                            # First prepare the copy if list is not empty
                            if value.count() > 0:
                                before_build_operations += " && cp "
                                # Now add all the files to the copy
                                for index in range(value.count()):
                                    before_build_operations += self.livecreatorConfig.get("main","hookdir") + "/" + str(value.item(index).text()) + " "
                                # Then the detination
                                before_build_operations += destDir[livecreatorOption]

            # Add a clean up script at the end of debootstrap
            file = os.getcwd() + "/specific_hooks/"
            destDir = "./config/chroot_local-hooks/"
            before_build_operations += " && cp " + file + "cleanup.sh " + destDir

            # Add both proxys
            httpproxy = self.livecreatorConfig.get("main","httpproxy")
            if len(httpproxy) != 0:
                opt_list += " --apt-http-proxy " + httpproxy + " "
            ftpproxy = self.livecreatorConfig.get("main","httpproxy")
            if len(ftpproxy) != 0:
                opt_list += " --apt-ftp-proxy " + ftpproxy + " "

            # Add the choosed mirrors
            mirror = self.livecreatorConfig.get("main","depurl")
            smirror = self.livecreatorConfig.get("main","secudepurl")
            opt_list += " --mirror-binary " + mirror + " --mirror-bootstrap " + mirror + " --mirror-boot " + mirror + " --mirror-binary-security " + smirror + " --mirror-chroot-security " + smirror

            # Add syslinux menu
            opt_list += " --syslinux-menu enabled"
            #Use apt instead of aptitude to avoid recommends
            opt_list += " --apt apt"
            # If needed, add boot options
            if bootappend != "":
                opt_list += " --bootappend-live " + '"' + bootappend + '"'

            # Prepare a shell script to launch the process
            # Choose the temporary directory and prepare a new directory
            tmpdir = self.livecreatorConfig.get("main","tmpdir")
            mkdir_cd = "mkdir -p %s/tmp_livecreator && cd %s/tmp_livecreator \n"%(tmpdir, tmpdir)
            # Image configuration
            lhconfig = "lh_config " + opt_list + opt_supp + "\n"
            # Middle operations
            before_build = 'echo "Do some LiveCreator specific operations"' + before_build_operations + "\n"
            # New kernel added
            if changekernel != '':
                before_build = before_build + changekernel + "\n"
            # Build command
            lhbuild = "lh_build | tee /var/log/livecreator.log\n"

            # write gen script in tmp file
            filename = "%slivecreator_live.sh"%tmpdir
            fw = open ( filename , "w")
            fw.writelines((mkdir_cd,lhconfig,before_build,lhbuild))
            fw.close()
            os.chmod(filename,0744)

            # clean tmp folder and umount
            cleanDialog = genimage.genCleanDialog(self.livecreatorConfig)
            cleanDialog.exec_()

            # Set error generation to zero
            self.procxterm.error = 0

            # Launch the configuration process
            self.procxterm.start(filename)
            # launch progress dialog and pass process as parameter to 
            # permit to stop it with the progress diaglog
            self.imageProgressdialog = genimage.genImageProgress(self.procxterm)
            self.imageProgressdialog.exec_()


    def setkbd(self, lang):
        """
            Return a well formated string to push in --bootappend-live lh_config option.
            Concerning the keyboard layout
        """
        if lang == "en":
            return " keyb=uk "
        elif lang == "fr":
            return " keyb=fr-latin1 "
        else:
            return " "


    def setlang(self, lang):
        """
            Return a well formated string to push in --bootappend-live lh_config option.
            Concerning the locale
        """
        if lang:
            lang = lang.split(" ")[0]
            return " locale=" + lang + " "


    def setdhcp(self, switch):
        """
            Return a well formated string to push in --bootappend-live lh_config option.
            Concerning the DHCP
        """
        if switch == "On":
            return ""
        elif switch == "Off" or switch == _("Choose during Live boot"):
            return " ip=frommedia "


    def interactiveMode(self, interMode, bootappend, before_build_operations, opt_list) :
        """
            Test if the interactive mode is already activated
            (by lang, keyboard, automount or dhcp)
            if not make some stuff to permit to 
            ask answers to the user
        """
        if interMode == 0:
            # Remember that the 
            #interactive mode is activated
            interMode = 1
            # We boot in runlevel 4
            bootappend += " 4 "
            #print opt_list
            #Test if --package is already set or not
            if str(opt_list).find("--packages") != -1:
                # If yes add dialog packages list
                opt_list = str(opt_list).replace('--packages "', '--packages " dialog')
            else:
                # If no add "--package" option and dialog
                opt_list += ''' --packages " dialog"'''
            # We need to copy some files
            file = os.getcwd() + "/specific_hooks/"
            destDir = "./config/chroot_local-includes/"
            before_build_operations += " && rsync -a " + file + "disable-services_overwrite/etc " + destDir
            before_build_operations += " && cp " + file + "disable-run4.sh ./config/chroot_local-hooks"
        return (interMode, bootappend, before_build_operations, opt_list)


    def state(self, argv):
        self.update()
        if argv == 0:
            print "stop"
            print self.procxterm.error
            # Test if
            if self.procxterm.error == 0:
                print 'stop normally'
                # Close progress dialog
                self.imageProgressdialog.closeDiag()
                # Enable the gen button
                self.GenImage.setDisabled(0)
                tmpdir = self.livecreatorConfig.get("main","tmpdir")
                # Detection de l'extension du fichier de sorti
                if self.allOptionsList[4][1] == 'usb-hdd':
                    extension = "img"
                elif self.allOptionsList[4][1] == 'iso':
                    extension = "iso"
                filename = "%s/tmp_livecreator/binary.%s"%(tmpdir,extension)
                # Test the image existence
                # If yes generation OK
                # if no generation failed
                if os.access(filename,os.F_OK):
                    # Generation OK
                    # Get time to create the image name
                    time = strftime("-%Y%m%d-%H%M", localtime())
                    # Create the image name
                    if self.xmlFile.getXMLValue("templatename"):
                        templatename = self.xmlFile.getXMLValue("templatename") + time
                    else:
                        templatename = "sans_nom" + time
                    # Move generated image to image folder
                    if filename.endswith(".img"):
                        shutil.move(filename, self.livecreatorConfig.get("main","imgdir") + templatename + ".img")
                    elif filename.endswith(".iso"):
                        shutil.move(filename, self.livecreatorConfig.get("main","imgdir") + templatename + ".iso")
                    self.testImgPage.initList()
                    # Final Pop up 
                    finish =  QtGui.QMessageBox( _("Information"),
                            _("Live generation has finish"),
                            QtGui.QMessageBox.Information, 
                            QtGui.QMessageBox.Yes, 
                            0, 
                            0 )
                    finish.setButtonText(QtGui.QMessageBox.Yes,"Ok")
                    finish.exec_()
                else:
                     # Generation failed
                    finish =  QtGui.QMessageBox( _("Failed generation"),
                            _("This generation has failed"),
                            QtGui.QMessageBox.Information, 
                            QtGui.QMessageBox.Yes, 
                            0, 
                            0 )
                    finish.setButtonText(QtGui.QMessageBox.Yes,"Ok")
                    finish.exec_()
            if self.procxterm.error == 1:
                print 'stop by user'
                # Close progress dialog
                self.imageProgressdialog.closeDiag()
                # Enable the gen button
                self.GenImage.setDisabled(0)
                self.testImgPage.initList()
                # Final Pop up 
                finish =  QtGui.QMessageBox( _("Cancelled generation"),
                        _('<font color="red"><b>You have cancelled image generation.</b></font><br>You can now relaunch the process to restart.'),
                        QtGui.QMessageBox.Critical, 
                        QtGui.QMessageBox.Yes, 
                        0, 
                        0 )
                finish.setButtonText(QtGui.QMessageBox.Yes,"Ok")
                finish.exec_()
        elif argv == 1:
            print 'start'
        elif argv == 2:
            print 'running'


    def generateSummary(self,allOptionsList):
        """
            Genereration summary from options
        """
        # Save options list for generating
        self.allOptionsList = allOptionsList
        # Check if generation is running
        if self.procxterm.state() == QtCore.QProcess.NotRunning:
            # Clean summarry
            self.summaryText.clear()
            # Write summary
            for options in self.allOptionsList:
                if type(options[1]) is type(QtCore.QString()) :
                    if not options[0].startswith("desc_"):
                        self.summaryText.insertHtml("<b>%s</b> : %s<br>"%(options[0],options[1]) )
                if type(options[1]) is type(QtGui.QListWidget()) :
                    self.summaryText.insertHtml("<b>%s</b> : <br>"%options[0] )
                    for index in range(options[1].count()):
                        self.summaryText.insertHtml("%s<br>"%options[1].item(index).text())
