#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys
from PyQt4 import QtCore, QtGui
from livecreator_misc import functions
import os
import subprocess as sp


class genCdDialog1(QtGui.QDialog):
    def __init__(self, devices, parent=None):
        """
            First dialog box. It permit to choose
            which device will be use to make the live
        """
        QtGui.QDialog.__init__(self, parent)

        self.setWindowTitle( _("Create a Live CD") )

        mediaLabel = QtGui.QLabel( _("Choose media where the Live will be writed on:") )

        okButton = QtGui.QPushButton( _("OK") )
        noButton = QtGui.QPushButton( _("Cancel") )

        self.typeDevCombo = QtGui.QComboBox()
        for device in devices:
            self.typeDevCombo.addItem(device)

        mainLayout = QtGui.QGridLayout()
        mainLayout.addWidget(mediaLabel,0,0,1,4)
        mainLayout.addWidget(self.typeDevCombo,1,1,1,2)
        mainLayout.addWidget(okButton,2,1,1,1)
        mainLayout.addWidget(noButton,2,2,1,1)

        QtCore.QObject.connect(okButton ,QtCore.SIGNAL('clicked()'),self.valid )
        QtCore.QObject.connect(noButton ,QtCore.SIGNAL('clicked()'),self.cancel )

        self.setLayout(mainLayout)

    def valid(self):
        """
            return the current selection and close box
        """
        self.done(self.typeDevCombo.currentIndex())

    def cancel(self):
        """
            return nothing and close the box
        """
        self.done(-1)


class genCdDialog2(QtGui.QMessageBox):
    def __init__(self, device, parent=None):
        """
            First warning message to be sure that
            user want to make the live
        """
        self.device = device

        QtGui.QMessageBox.__init__(self, _("Create the Live"),
                                    "",
                                    QtGui.QMessageBox.Warning,
                                    QtGui.QMessageBox.Yes,
                                    QtGui.QMessageBox.No,
                                    0)
        self.setWindowTitle( _("Create a Live CD") )
        self.setButtonText(QtGui.QMessageBox.Yes,_("&Yes") )
        yesButton = self.button(QtGui.QMessageBox.Yes)
        self.setButtonText(QtGui.QMessageBox.No,_("&No") )
        self.setText( _("Please insert a blank media.<br>Are you sure that you want generate a CD/DVD on the media <b>%s</b>?<br>This operation takes some time, please be patient") %device)

        #Signals
        QtCore.QObject.connect(yesButton ,QtCore.SIGNAL('clicked()'),self.valid)


    def checkDisk(self):
        """
            For CD/DVD check, if a cd or dvd is in the device
        """
        cmd = "wodim -atip dev=/dev/%s"%self.device
        checkCd = sp.Popen (cmd,
                     bufsize = 1,
                     shell = True,
                     stdout = sp.PIPE,
                     stderr = sp.STDOUT)

        if checkCd.stdout.readlines()[0] == "wodim: Cannot load media with this drive!\n":
            # No CD/DVD in the device or device not closed
            return False
        else:
            # CD/DVD Present un the device
            return True
        checkCd.stdout.close()


    def valid(self):
        """
            Made checks before close the box
        """
        # If yes button is clicked
        if self.checkDisk():
            self.done(QtGui.QMessageBox.Yes)
        else:
            # Open the CD/DVD player
            cmd = 'eject /dev/%s'%self.device
            eject = sp.Popen(cmd,
                                bufsize = 1,
                                shell = True,
                                stdin = sp.PIPE,
                                stdout = sys.stdout,
                                stderr = sys.stderr,
                                universal_newlines = True)
            self.done(-1)


class genCdDialog3(QtGui.QMessageBox):
    def __init__(self,device, parent=None):
        """
            Second and last message to cancel the
            making of the live
        """
        QtGui.QMessageBox.__init__(self, _("Create the Live"),
                                    "",
                                    QtGui.QMessageBox.Warning,
                                    QtGui.QMessageBox.Yes,
                                    QtGui.QMessageBox.No,
                                    0)
        self.setWindowTitle( _("Create the Live CD") )
        self.setButtonText(QtGui.QMessageBox.Yes, _("&Yes") )
        self.setButtonText(QtGui.QMessageBox.No, _("&Non") )
        self.setText( _("Are you <b>REALLY</b> sure about generate the CD/DVD on the media <b>%s</b>?") %device)


class genCdProgress(QtGui.QProgressDialog):
    def __init__(self, parent=None):
        """
            Progress bar dialog for bruning CD/DVD
        """
        QtGui.QProgressDialog.__init__(self, parent)
        self.setWindowTitle( _("Create a Live CD") )
        self.setLabelText( _("CD/DVD creation") )
        cancelButton = QtGui.QPushButton("")
        self.setCancelButton(cancelButton)
        cancelButton.hide()
        self.setAutoClose(0)
        self.setAutoReset(0)
        # Set Progress bar
        #self.bar = QtGui.QProgressBar()
        ##self.setBar(self.bar)
        #self.bar.setValue(10)


    def closeEvent(self, event):
        """
            Don"t close progress bar when
            user click on the X (close)
        """
        event.ignore()


class genCdSummary(QtGui.QMessageBox):
    def __init__(self, returnProc, parent=None):
        """
            Generate a box which says if the generation
            seems to be ok or not
        """
        QtGui.QMessageBox.__init__(self)
        self.setWindowTitle( _("Create a Live CD") )
        # CD/DVD box
        if returnProc == 0:
            self.setIcon(QtGui.QMessageBox.Information)
            self.setText( _("Live CD/DVD creation is a success.") )
        elif returnProc == 1:
            self.setIcon(QtGui.QMessageBox.Critical)
            self.setText( _("Live CD/DVD creation has failed.") )
