#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from __future__ import division
import sys
from PyQt4 import QtCore, QtGui
from livecreator_misc import functions
import os, time,os.path
import fnmatch
import genmediadialog as gen
import genusbdialog as genusb
import subprocess as sp
from livecreator_misc import tips


class TestImgPage(QtGui.QWidget):
    def __init__(self, livecreatorConfig, parent=None):
        QtGui.QWidget.__init__(self, parent)

        self.livecreatorConfig = livecreatorConfig
        # self.devType Define which type (CD/DVD or USB-HDD)
        # of media will be create
        self.devType = ""
        # self.device Define which device will be use to
        # create the media (ex: /dev/sdb, /dev/cdrom)
        self.device =""
        # Size in byte of the image selected
        self.totalSizeInByte = 0
        # Temp progress bar value for usb media
        self.progressUsb = 0.000000
        #Image management
        addImages = QtGui.QPushButton( _("Import an existing image") )
        addImages.setToolTip(tips.allTips().test_import)
        self.delImages = QtGui.QPushButton( _("Delete this image") )
        self.delImages.setToolTip(tips.allTips().test_del)
        self.delImages.setDisabled(1)
        imageGroup = QtGui.QGroupBox( _("Images list") )
        self.imageList = QtGui.QListWidget()
        self.testImg = QtGui.QPushButton( _("Test this image") )
        self.testImg .setToolTip(tips.allTips().test_image)
        self.testImg.setDisabled(1) 
        self.genMedia = QtGui.QPushButton( _("Create the Live") )
        self.genMedia.setToolTip(tips.allTips().test_gen)
        self.genMedia.setDisabled(1) 
        imageLayout = QtGui.QGridLayout()
        imageLayout.addWidget(addImages,1,0)
        imageLayout.addWidget(self.delImages,1,1)
        imageLayout.addWidget(self.imageList,0,0,1,0)
        imageLayout.addWidget(self.testImg,2,0)
        imageLayout.addWidget(self.genMedia,2,1)
        imageGroup.setLayout(imageLayout)

        # File loaded is set to empty string
        self.filename = ""
        # Create a  proc to put the qemu core inside
        self.procqemu = QtCore.QProcess()
        infoGroup = QtGui.QGroupBox( _("About this image") )
        infolayout = QtGui.QVBoxLayout()
        self.nameImg = QtGui.QLabel( _("<b>Name : </b>") )
        self.typeImg = QtGui.QLabel( _("<b>Typz : </b>") )
        self.sizeImg = QtGui.QLabel( _("<b>Size: </b>") )
        self.dateImg = QtGui.QLabel( _("<b>Creation date: </b>") )
        infolayout.addWidget(self.nameImg,0)
        infolayout.addWidget(self.sizeImg,1)
        infolayout.addWidget(self.typeImg,2)
        infolayout.addWidget(self.dateImg,3)
        infoGroup.setLayout(infolayout)
        infoGroup.setFixedWidth(300)
        #infoGroup.setFixedWidth(450)
        testGroup = QtGui.QGroupBox(self.tr( _("Avalaible image choice") ))
        testlayout = QtGui.QGridLayout()

        # This button will permit to launch a qemu directly on the real device
        self.testMedia = QtGui.QPushButton( _("Test a live") )
        self.testMedia.setToolTip(tips.allTips().test_live)
        self.testMedia.setIcon(QtGui.QIcon("images/test_live.png"))

        testlayout.addWidget(imageGroup,0,0,2,3)
        testlayout.addWidget(infoGroup,0,3,1,2)
        testlayout.addWidget(self.testMedia,1,3,1,2)
        testGroup.setLayout(testlayout)


        mainLayout = QtGui.QVBoxLayout()
        mainLayout.addWidget(testGroup)

        # Generation proccess
        self.procxterm = QtCore.QProcess()
        # Return value of the process
        # All it's ok  == 0
        # An error     == 1
        self.returnProc = 1
        # Burn progressbar
        self.burnProgressBar = gen.genCdProgress()
        # Usb progressbar
        self.usbProgressBar = genusb.genUsbProgress()
        # Process signals
        self.connect(self.procxterm, QtCore.SIGNAL('readyReadStandardOutput()'), self.readOutput)
        self.connect(self.procxterm, QtCore.SIGNAL('readyReadStandardError()'), self.readErrorOutput)
        self.connect(self.procxterm, QtCore.SIGNAL('stateChanged (QProcess::ProcessState)'), self.stateProcess)


        #Signals
        QtCore.QObject.connect(addImages, QtCore.SIGNAL('clicked()'), self.addImage )
        QtCore.QObject.connect(self.delImages, QtCore.SIGNAL('clicked()'), self.delImage )
        QtCore.QObject.connect(self.testImg, QtCore.SIGNAL('clicked()'), self.testLaunch )
        QtCore.QObject.connect(self.genMedia,
                                    QtCore.SIGNAL('clicked()'), self.preGenMediaLaunch )
        QtCore.QObject.connect(self.imageList, QtCore.SIGNAL('itemSelectionChanged()'), self.imgInfo)
        QtCore.QObject.connect(self.testMedia, QtCore.SIGNAL('clicked()'), self.testMediaLaunch)

        self.setLayout(mainLayout)
        self.initList()


    def initList(self):
        """
            Create an images list
        """
        self.imageList.clear()
        imgdir = self.livecreatorConfig.get("main", "imgdir")
        ilist = []
        for image in functions.readDir(imgdir, "*.iso"):
            self.imageList.addItem(imgdir+image)

        for image in functions.readDir(imgdir, "*.img"):
            self.imageList.addItem(imgdir+image)


    def addImage(self):
        file = QtGui.QFileDialog.getOpenFileName(self, self.tr( _("Import an image") ),
                        QtCore.QString(self.livecreatorConfig.get("main","imgdir")),"Img (*.iso *.img);;All Files (*)")
        if file:
            self.imageList.addItem(str(file) )


    def imgInfo(self):
        # Choosing the image name in the list
        # self.imageList.selectedItems() is a list with selected image as fisrt element
        if len(self.imageList.selectedItems()) > 0:
            self.filename = self.imageList.selectedItems()[0].text()[:]
            stat = os.stat(self.filename)
            date = time.localtime(stat[9])
            self.totalSizeInByte = stat[6]
            self.size = functions.format_size(self.totalSizeInByte)
            # get totalSize for generation progress bar
            self.totalSize = int(self.size.split(".")[0])
            date = functions.format_date(date,"date")
            self.filetype = functions.format_live(os.path.basename(str(self.filename)) )
            self.nameImg.setText( _("<b>Name: </b>%s") % os.path.basename(str(self.filename)) ) 
            self.typeImg.setText( _("<b>Type: </b>%s") % self.filetype )
            self.sizeImg.setText( _("<b>Size: </b>%s") % self.size)
            self.dateImg.setText( _("<b>Creation date: </b>%s / %.2d / %s") % (date[0],date[1],date[2]) )
            self.testImg.setDisabled(0)
            self.genMedia.setDisabled(0)
            self.delImages.setDisabled(0)
        else:
            self.nameImg.setText( _("<b>Name: </b>") )
            self.typeImg.setText( _("<b>Type: </b>") )
            self.sizeImg.setText( _("<b>Size: </b>") )
            self.dateImg.setText( _("<b>Creation date: </b> ") )
            self.testImg.setDisabled(1)
            self.genMedia.setDisabled(1)
            self.delImages.setDisabled(1)


    def delImage(self):
        """
            Delete a image in image list and ONLY in image folder
        """
        imageDir = self.livecreatorConfig.get("main","imgdir")
        if self.imageList.currentItem():
            # Item seleted
            itemSeleted = self.imageList.currentItem().text()
            # Check if the image is in the image folder
            if str(itemSeleted).startswith(imageDir):
                textBox = _("Delete the image %s?<br>< font color=red><b>The image file will be deleted from hard disk too.</b></font>") %itemSeleted
            else:
                textBox = _("Delete the image %s ?") %itemSeleted
            # Question box
            question = QtGui.QMessageBox( _("Image suppression"),
                textBox,
                QtGui.QMessageBox.Critical, 
                QtGui.QMessageBox.Yes, 
                QtGui.QMessageBox.No, 
                0 )
            question.setButtonText(QtGui.QMessageBox.Yes, _("&Yes") )
            question.setButtonText(QtGui.QMessageBox.No, _("&No'") )
            if question.exec_() == QtGui.QMessageBox.Yes:
                # Check if the image is in the image folder
                if str(itemSeleted).startswith(imageDir):
                    # Delete from template folder
                    os.remove( "%s"%(itemSeleted) )
                # reload template list
                index = self.imageList.row(self.imageList.currentItem())
                self.imageList.takeItem(index)
                #self.initList()


    def testLaunch(self):
        """
            Launch a qemu with the image name as parameter.
        """
        # check if the file is empty
        if self.totalSizeInByte == 0:
            critical = QtGui.QMessageBox( _("Image error"),
                _("Image is empty.\nImpossible to test anything"),
                QtGui.QMessageBox.Critical, 
                QtGui.QMessageBox.Yes,
                0,
                0 )
            critical.setButtonText(QtGui.QMessageBox.Yes,"Ok")
            critical.exec_()
        # Set the device type of the live
        elif self.filetype == "CD/DVD" or self.filetype == "USB-HDD":
            # Set cdrom or iso
            if self.filetype == "CD/DVD":
                self.opt_list = ["-cdrom", self.filename]
            elif self.filetype == "USB-HDD":
                self.opt_list = ["-hda", self.filename]
            self.procqemu.start('qemu', self.opt_list)
            #test = testImageDialog()
            #testReturn = test.exec_()
            ## i368
            #if testReturn == 0:
                #self.procqemu.start('qemu', self.opt_list)
            ## Amd64
            #elif testReturn == 1:
                #self.procqemu.start('qemu-system-x86_64', self.opt_list)
            ## Powerpc
            #elif testReturn == 2:
                #self.procqemu.start('qemu-system-ppc', self.opt_list)
            #else:
                ## do nothing
                #return

        else:
            critical = QtGui.QMessageBox( _("Image error"),
                _("Image format unknown.\nImpossible to test anything."),
                QtGui.QMessageBox.Critical, 
                QtGui.QMessageBox.Yes,
                0,
                0 )
            critical.setButtonText(QtGui.QMessageBox.Yes,"Ok")
            critical.exec_()


    def testMediaLaunch(self):
        """
            Launch a qemu with the media as parameter.
        """
        # Find all devices
        devices = self.existingDevices("cd") + self.existingDevices("usb") 
        test = testLiveDialog(devices)
        device = test.exec_()
        print devices[device]
        # Set qemu option in function of selected device
        if devices[device].startswith("cd"):
            self.opt_list = ["-cdrom", "/dev/"+devices[device]]
        elif devices[device].startswith("sd"):
            self.opt_list = ["-hda", "/dev/"+devices[device]]
        else:
            # do nothing
            return
        # Set qemu architecture
        if device != -1:
            self.procqemu.start('qemu', self.opt_list)
            #test = testImageDialog()
            #arch = test.exec_()
            ## i368
            #if arch == 0:
                #self.procqemu.start('qemu', self.opt_list)
            ## Amd64
            #elif arch == 1:
                #self.procqemu.start('qemu-system-x86_64', self.opt_list)
            ## Powerpc
            #elif arch == 2:
                #self.procqemu.start('qemu-system-ppc', self.opt_list)
            #else:
                ## do nothing
                #return


    def preGenMediaLaunch(self):
        """
            Show warning boxes before to generate the media.
        """
        if len(self.filename):
            # check if the file is empty
            if self.totalSizeInByte == 0:
                critical = QtGui.QMessageBox( _("Image error"),
                    _("Empty file.\nImpossible to test anything."),
                    QtGui.QMessageBox.Critical, 
                    QtGui.QMessageBox.Yes,
                    0,
                    0 )
                critical.setButtonText(QtGui.QMessageBox.Yes,"Ok")
                critical.exec_()
                return
            # Set the device type of the live
            if self.filetype == "CD/DVD":
                self.devType = "cd"
            elif self.filetype == "USB-HDD":
                self.devType = "usb"
            # Warning messages
            devices = self.existingDevices(self.devType)
            if self.devType == "cd":
                genMediaDial1 = gen.genCdDialog1(devices)
                device = genMediaDial1.exec_()
                if device >= 0:
                # For cd/dvd , checking cdrom device and eject it
                # to put the media in the good device
                    self.device = devices[device]
                    genMediaDial2 = gen.genCdDialog2( self.device)
                    # check if a player is in the CD/DVD player
                    box2 = -1
                    while box2 == -1:
                        # Show the second box while a CD/DVD is in the device
                        # Or user click on "NO"
                        box2 = genMediaDial2.exec_()
                    if box2 == QtGui.QMessageBox.Yes:
                        # Show last box
                        genMediaDial3 = gen.genCdDialog3(self.device)
                        if genMediaDial3.exec_() == QtGui.QMessageBox.Yes:
                            self.genCdLaunch(self.device)
                # For USB 
            if self.devType == "usb":
                genMediaDial1 = genusb.genUsbDialog1()
                box1 = -1
                while box1 == -1:
                    box1 = genMediaDial1.exec_()
                if box1 == QtGui.QMessageBox.Yes:
                    self.device = genMediaDial1.device
                    self.deviceName = genMediaDial1.deviceName
                    genUsbDial2 = genusb.genUsbDialog2(self.deviceName)
                    if genUsbDial2.exec_() == QtGui.QMessageBox.Yes:
                        genUsbDial3 = genusb.genUsbDialog3(self.device, self.deviceName)
                        if genUsbDial3.exec_() == QtGui.QMessageBox.Yes:
                            self.genCdLaunch(self.device)


    def genCdLaunch(self,device):
        """
            Launch a process to generate CD/DVD
        """
        if self.devType == "cd":
            # Burn CD/DVD with wodim
            # wodim dev=/dev/cdrom self.filename
            burn =  '''wodim -v -eject dev=/dev/%s -data "%s"'''%(device,self.filename)
            # Launch the generate process
            self.procxterm.start(burn)
            self.burnProgressBar.exec_()
        elif self.devType == "usb":
            self.progressUsb = 0
            # Create USB key with sdd => more faster than dd
            # and can catch an output
            usbWrite =  '''sdd -pg if="%s" of=/dev/%s'''%(self.filename,device)
            # Launch the generate process
            self.procxterm.start(usbWrite)
            self.usbProgressBar.exec_()


    def stateProcess(self, argv):
        """
            Watch state of the process
        """
        self.update()
        if argv == 0:
            # Send returnProc and device type to 
            # the media generation summary box
            if self.devType == "cd":
                gen.genCdSummary(self.returnProc).exec_()
                self.burnProgressBar.cancel()
            if self.devType == "usb":
                genusb.genUsbSummary(self.returnProc).exec_()
                self.usbProgressBar.cancel()
            self.genMedia.setDisabled(0) 
        elif argv == 1:
            #self.summaryText.clear()
            print 'start'
            self.genMedia.setDisabled(1)
        elif argv == 2:
            print 'running'


    def readErrorOutput(self):
        """
            Read the process errors Outputs
        """
        # Read errors outputs from bruning process
        if self.devType == "cd":
            errorOutput = str(QtCore.QString(self.procxterm.readAllStandardError()))
            print errorOutput
            if ( errorOutput.startswith("wodim: Cannot load media with this drive") or
                 errorOutput.startswith("wodim: Device or resource busy.")
                ):
                # ERROR ======> CD/DVD not empty
                # Set the process value to 0
                self.returnProc = 1
                # Stop the process
                self.procxterm.close()

        # Read errors outputs from usb making process
        elif self.devType == "usb":
            errorOutput = str(QtCore.QString(self.procxterm.readAllStandardError()))
            if  errorOutput.find("sdd: Wrote") != -1:
                # Check if all bytes are wrote
                if errorOutput.find(str(self.totalSizeInByte)) != -1:
                    # if yes, USB key created successfully
                    self.returnProc = 0
                    return
                else:
                    # if not, USB key did'nt create successfully
                    self.returnProc = 1
                    return
            # Each point "." in errorOutput is a records
            # each record is 512ko
            # So we count how many point is shown
            # to calculate the progress percent.
            pointCount = errorOutput.count(".")
            if pointCount >= 1:
                # if the error report records ...
                # ... Calc the percent progress value
                totalRecords = self.totalSizeInByte/512
                percent="%.10f"%(pointCount*100/totalRecords)
                self.progressUsb += float(percent)
                # And set the progress bar value
                self.usbProgressBar.setValue(self.progressUsb)

    def readOutput(self):
        """
            Read the process standards Outputs
        """
        # Read standards outputs from bruning process
        if self.devType == "cd":
            # Get ouput
            output =  str(self.procxterm.readAllStandardOutput())
            if output.startswith('Fixating time'):
                self.returnProc = 0
            # Cut output string
            templist = output.split(" ")
            try :
                # Try to find progress value
                progressValueStr = templist[templist.index("of")-1]
                progressValue = int(progressValueStr)
                #print type(progressValue)
                # Translate it to percent
                percentValue = progressValue*100/self.totalSize
                # Set the value of progressBar
                self.burnProgressBar.setValue(percentValue)
                # Test if percent Value = 100%
                # If yes, change box label
            except Exception,e:
                return
            # Change text Label in progress box
            if percentValue == 100:
                self.burnProgressBar.setLabelText( _("CD/DVD Finalization") )

        # Read standards outputs from usb making process
        elif self.devType == "usb":
            output =  str(self.procxterm.readAllStandardOutput())


    def existingDevices(self,devtype):
        """
            list existing Devices in function of type :
            usb or cdrom
        """
        devices = []
        # Test type
        if devtype == "cd":
            devices = fnmatch.filter(os.listdir("/dev/"), "cdrom*")
        elif devtype == "usb":
            # List all devices depending of device type
            devices = fnmatch.filter(os.listdir("/dev/"), "sd??")
            # read fstab
            fstab = open("/etc/fstab","r")
            #print fstab.readline()
            for line in fstab.readlines():
                for device in devices:
                    if line.find(device) != -1:
                      devices.remove(device)
        return devices


class testImageDialog(QtGui.QDialog):
    def __init__(self, parent=None):
        """
            First dialog box. It permit to choose
            which architecture will be use to test the live
            with qemu
        """
        QtGui.QDialog.__init__(self, parent)


        self.setWindowTitle( _("Test this image") )
        #TODO: normaly impossible case... Or with external image maybe ? To check
        archLabel = QtGui.QLabel( _("Choose  th image architecture:") )
        self.archCombo = QtGui.QComboBox()
        self.archCombo.addItem("i386")
        self.archCombo.addItem("amd64")
        self.archCombo.addItem("powerpc")

        okButton = QtGui.QPushButton( _("OK") )
        noButton = QtGui.QPushButton( _("Cancel") )

        mainLayout = QtGui.QGridLayout()
        mainLayout.addWidget(archLabel,0,0,1,4)
        mainLayout.addWidget(self.archCombo,1,1,1,2)
        mainLayout.addWidget(okButton,2,1,1,1)
        mainLayout.addWidget(noButton,2,2,1,1)

        QtCore.QObject.connect(okButton ,QtCore.SIGNAL('clicked()'),self.valid )
        QtCore.QObject.connect(noButton ,QtCore.SIGNAL('clicked()'),self.cancel )

        self.setLayout(mainLayout)

    def valid(self):
        """
            return the current selection and close box
        """
        self.done(self.archCombo.currentIndex())

    def cancel(self):
        """
            return nothing and close the box
        """
        self.done(-1)


class testLiveDialog(QtGui.QDialog):
    def __init__(self, devices=None, parent=None):
        """
            First dialog box. It permit to choose
            which device will be use to test the live
            with qemu
        """
        QtGui.QDialog.__init__(self, parent)

        self.devices = devices

        self.setWindowTitle( _("Test this Live") )
        deviceLabel = QtGui.QLabel( _("Choose Live media") )
        self.deviceCombo = QtGui.QComboBox()
        for device in devices:
            self.deviceCombo.addItem(device)

        okButton = QtGui.QPushButton( _("OK") )
        noButton = QtGui.QPushButton( _("Cancel") )

        mainLayout = QtGui.QGridLayout()
        mainLayout.addWidget(deviceLabel,0,0,1,4)
        mainLayout.addWidget(self.deviceCombo,1,1,1,2)
        mainLayout.addWidget(okButton,2,1,1,1)
        mainLayout.addWidget(noButton,2,2,1,1)

        QtCore.QObject.connect(okButton ,QtCore.SIGNAL('clicked()'),self.valid )
        QtCore.QObject.connect(noButton ,QtCore.SIGNAL('clicked()'),self.cancel )

        self.setLayout(mainLayout)

    def valid(self):
        """
            return the current selection and close box
        """
        self.done(self.deviceCombo.currentIndex())

    def cancel(self):
        """
            return nothing and close the box
        """
        self.done(-1)
