#!/usr/bin/env python

# -*- coding: utf-8 -*-

import xml.etree.ElementTree as et
import os.path
import re

class ConfigFile():

    def __init__(self):
        """
            Create an oject to handle configurations.
            The object is build around two main contain :
            - The liveconfiguration (usb/CD, Timeout, lang, ...)
            - The packets selected
            The object is saved in XML and keept in memory as xml tree.
        """
        self.configTree = et.Element("config", name= "none")


    def loadRawFile(self, filename, path="/usr/share/livehelper/lists"):
        """
            Build XML profile from Debian-Live profile file
        """
        profileslist = []
        packages = []

        # Open  and read file
        try:
            file = open(filename, "r")
            content = file.readlines()
        except Exception, e:
            print "Ouverture du fichier %s : %s" %(filename, str(e))
        finally:
            file.close()

        # Read each line and do actions
        for line in content:
            line = line[:-1]
            # Obtain Profile name
            if line.startswith('## LH:'):
    		    rootline = line
            # Included profiles
            elif line.startswith('#<include>'):
                profileslist.append(line.split(" "))
            # List of depends
            elif not line.startswith("#") and len(line) != 0:
                for package in line.split(" "):
                    packages.append(package)

        # Creating Root
        root = et.Element("profile", filename= os.path.basename(filename),
                                         template=rootline)

        # Creating included profiles
        for profile in profileslist:
            temp = self.loadRawFile(path + "/" + profile[1])
            root.append(temp)
        # included packages
        packagesTree = et.SubElement(root,'packages')
        for package in packages:
            packTree = et.SubElement(packagesTree,'package')
            packTree.text = package

        # Change the profileTree setting this as the new one
        # Only the last one will be pushed as profiletree
        # Previous recursive set profile operations will be cancelled
        self.profileTree = root
        # This line is neccessary to return the tree during recursives calls
        return root


    def loadXML(self, filename, path="."):
        """
            Build Profile from XML
        """
        # Open  and read file
        try:
            self.profileTree = et.parse(path + "/" + filename).getroot()
        except Exception, e:
            print "Erreur durant l'interpretation du fichier %s : %s" %(path + "/" + filename, str(e))

    def toFile(self, filename, path="."):
        """
            Dump Profile as a debian live profile file
            Note that it will create a file only for top level template
        """
        # Build header
        header = "# " + path + "/" + filename + " - package list for livehelper(7)\n"
        header += "\n"
        # This is the first line of config, to obtain '## LH: ????'
        header += self.profileTree.items()[0][1] + "\n"
        # Obtain includes
        list = [ profile for profile in self.profileTree.getchildren() if profile.tag == "profile"]
        for include in list:
            header += "#<include> " + include.attrib["filename"] + "\n"
        header += "\n"

        # Get all packages names
        plist = []
        for child in self.profileTree.getchildren():
            if child.tag == "packages":
                for package in child.getchildren():
                    plist.append(package.text)
        try:
            file = open(path + "/" + filename, "w")
            # Write header
            file.write(header)
            # Write all package in this template
            for package in plist:
                file.write(package + "\n")
        except Exception, e:
            print "Erreur de sauvegarde du fichier %s : %s" %(path + "/" + filename, str(e))


    def toXML(self, filename, path="."):
        """
            Dump xml representation of this config in a file
        """
        # Open  and read file
        try:
            file = open(path + "/" + filename, "w")
            file.write( et.tostring(self.profileTree) )
        except Exception, e:
            print "Erreur de sauvegarde du fichier %s : %s" %(path + "/" + filename, str(e))
        finally:
            file.close()


#    def getPackages(self, tree=self.profileTree):
#        list = []
#        plist = [ packages for packages in tree.getchildren() if packages.tag == "packages"]
#        if len(plist) != 0:
#            for packageset in plist:
#                for package in packageset.getchildren():
#                    list.append(package.text)
#        else:
#            return a
#&        return list

#config = ConfigFile()
#config.loadRawFile("/usr/share/livehelper/lists/xfce-junior")
#config.toXML("plip.xml")
#config.loadXML("plip.xml")
#config.toFile("plip.raw", "/tmp")

