#!/usr/bin/env python
# -*- coding: UTF-8 -*-


import xml.etree.ElementTree as et
import os.path
import re

class xmlTemplate():

    def __init__(self):
        """
            Create an oject to handle configurations.
            The object is build around two main contain :
            - The liveconfiguration (usb/CD, Timeout, lang, ...)
            - The packets selected
            The object is saved in XML and keept in memory as xml tree.
        """
        self.profileTree = et.Element("templatefile")


    def loadXML(self, filename, path="."):
        """
            Build Profile from XML
        """
        # Open  and read file
        try:
            self.profileTree = et.parse(path + "/" + filename).getroot()
        except Exception, e:
            print "Erreur durant l'interpretation du fichier %s : %s" %(path + "/" + filename, str(e))


    def toXML(self, filename, path="."):
        """
            Dump xml representation of this config in a file
        """
        # Open and write file
        try:
            file = open(path + "/" + filename, "w")
            file.write( et.tostring(self.profileTree) )
        except Exception, e:
            print "Erreur de sauvegarde du fichier %s : %s" %(path + "/" + filename, str(e))
        finally:
            file.close()


# All get methods
    def getPackagesList(self, origin="upstream"):
        """
            USED BY LiveCreator_QT
            Return the list of selected packages.
            Result is a list of packages as elements.
            The origin can be "upstream" for officials packages or "local".
        """
        # Fisrt check if package elements exists
        ltype_element = self.profileTree.find("packages")
        if ltype_element:
            # Check now of subelement "origin" exist either
            origin_element = ltype_element.find(origin)
            if origin_element:
                # Both exist, let's return a packages list !
                plist = origin_element.findall("package")
                return [ packet.text for packet in  plist ]
        return []


    def getHooksList(self, origin="chroot"):
        """
            USED BY LiveCreator_QT
            Return the list of selected hooks.
            Result is a list of scripts as elements.
            The origin can be "chroot" or "binary".
        """
        # Fisrt check if hooks element exists
        ltype_element = self.profileTree.find("hooks")
        if ltype_element:
            # Check now of subelement "origin" exist either
            origin_element = ltype_element.find(origin)
            if origin_element:
                # Both exist, let's return a scripts list !
                slist = origin_element.findall("script")
                return [ script.text for script in  slist]
        return []


####
####
####
    def getSourcesList(self, origin="source"):
        """
            USED BY LiveCreator_QT
            Return the list of selected sources packages.
            Result is a list of scripts as elements.
            The origin can be only "source".
        """
        ltype_element = self.profileTree.find("sources")
        if ltype_element:
            # Check now of subelement "origin" exist either
            origin_element = ltype_element.find(origin)
            if origin_element:
                # Both exist, let's return a scripts list !
                slist = origin_element.findall("source")
                return [ source.text for source in  slist]
        return []


    def emptySourcesList(self, origin="source"):
        """
            Empty the sources list on the xml tree
        """
        sources = self.profileTree.find("sources")
         # We can clear only an existent element
        if sources :
            sorigin = sources.find(origin)
            if sorigin :
                sorigin.clear()


    def setSourcesList(self, slist, origin="source"):
        """
            USED BY LiveCreator_QT
            Set the sources list on the xml tree
        """
        self.emptySourcesList(origin)
        for source in slist:
            self.addSource(source, origin)


    def addSource(self, sname, origin="source"):
        """
            Add a source in the existing list.
        """
        source = et.Element("source")
        source.text = sname

        # If source element d'ont exist, we must create it.
        if not self.profileTree.find("sources"):
            self.profileTree.append(et.Element("sources"))
        sources = self.profileTree.find("sources")
        # Same if origin does not exist
        if not sources.find(origin):
            sources.append(et.Element(origin))
        sources.find(origin).append(source)
###
###
###


    def getXMLValue(self, vname):
        """
            USED BY LiveCreator_QT
            Return the string associate to the "vname" XML field.
            For example : if vname is crypt_level the return can be 128
        """
        element = self.profileTree.find(vname)
        if not element is None and not element.text is None:
            return element.text
        return ""


# All set methods
    def emptyPackagesList(self, origin="upstream"):
        """
            Empty the packages list on the xml tree
        """
        packages = self.profileTree.find("packages")
         # We can clear only an existent element
        if packages :
            porigin = packages.find(origin)
            if porigin :
                porigin.clear()

    def setPackagesList(self, plist, origin="upstream"):
        """
            USED BY LiveCreator_QT
            Set the packages list on the xml tree
        """
        self.emptyPackagesList(origin)
        for package in plist:
            self.addPackage(package, origin)


    def addPackage(self, pname, origin="upstream"):
        """
            Add a package in the existing list.
        """
        package = et.Element("package")
        package.text = pname

        # If packages element d'ont exist, we must create it.
        if not self.profileTree.find("packages"):
            self.profileTree.append(et.Element("packages"))
        packages = self.profileTree.find("packages")
        # Same if origin does not exist
        if not packages.find(origin):
            packages.append(et.Element(origin))
        packages.find(origin).append(package)


    def emptyHooksList(self, origin="chroot"):
        """
            Empty the hooks list on the xml tree
        """
        hooks = self.profileTree.find("hooks")
        # We can clear only an existent element
        if hooks :
            horigin = hooks.find(origin)
            if horigin :
                horigin.clear()

    def setHooksList(self, slist, origin="chroot"):
        """
            USED BY LiveCreator_QT
            Set the hooks list on the xml tree
            slist is a script list (whith entire path)
        """
        self.emptyHooksList(origin)
        for script in slist:
            self.addHook(script, origin)


    def addHook(self, sname, origin="chroot"):
        """
            Add a script in the existing hooks list.
        """
        hookscript = et.Element("script")
        hookscript.text = sname

        # If hooks element d'ont exist, we must create it.
        if not self.profileTree.find("hooks"):
            self.profileTree.append(et.Element("hooks"))
        hooks = self.profileTree.find("hooks")
        # Same if origin does not exist
        if not hooks.find(origin):
            hooks.append(et.Element(origin))
        hooks.find(origin).append(hookscript)

    def valueExist(self, vname):
        """
            Return "True" if the value exist, None else.
            To check the existence, we check if the value name exist in
            the XML tree.
        """
        if not self.profileTree.find(vname) is None:
            return True
        else:
            return None


    def setXMLValue(self, vname, value):
        """
            USED BY LiveCreator_QT
            Put a "value" in the XML tree as "vname" (vname can be :
            type, hostname,  ...).
        """
        if value:
            if self.valueExist(vname):
                self.setExistentValue(vname, value)
            else:
                self.setNewValue(vname, value)
        else:
            if self.valueExist(vname):
                self.delExistentValue(vname)


    def setExistentValue(self, vname, value):
        """
            Change an existing value in the XML tree.
        """
        self.profileTree.find(vname).text = value

    def setNewValue(self, vname, value):
        """
            Create new XML element and put it in the tree
        """
        elem = et.Element(vname)
        elem.text = value
        self.profileTree.insert(0,elem)

    def delExistentValue(self, vname):
        """
            Delete an existent value.
        """
        todel = self.profileTree.find(vname)
        todel.clear()
