#!/bin/sh

######################################################################
# create_usplash_theme                                Michel DEPEIGE #
# 27/03/2008                             michel.depeige@linagora.com #
#                                                                    #
# Description:                                                       #
# Cre un thme usplash a partir d'un fichier image (reconnu par     #
# imagemagick ) et de usplash-theme-ubuntu_0.17.tar.gz               #
#                                                                    #
# IN : <fichier.foo>                                                 #
# OUT : /tmp/usplash-theme-LiveCreator.so                            #
#                                                                    #
#                                                                    #
#--------------------------------------------------------------------#
# Copyright (C) Michel DEPEIGE <mdepeige@linagora.com>               #
#               Erwan LE GALL <elegall@linagora.com>                 #
# Copyright (C) LINAGORA <http://www.linagora.com>                   #
#                                                                    #
#--------------------------------------------------------------------#
# License: GPLv3                                                     #
#--------------------------------------------------------------------#
# This program is free software; you can redistribute it and/or      #
# modify it under the terms of the GNU General Public License        #
# as published by the Free Software Foundation; either version 2     #
# of the License, or (at your option) any later version.             #
#                                                                    #
# This program is distributed in the hope that it will be useful,    #
# but WITHOUT ANY WARRANTY; without even the implied warranty of     #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      #
# GNU General Public License for more details.                       #
#                                                                    #
# GPL License: http://www.gnu.org/licenses/gpl.txt                   #
#                                                                    #
#--------------------------------------------------------------------#
# Please visit http://www.linagora.org for other contributions       #
#                                                                    #
#--------------------------------------------------------------------#
# Versions            :                                              #
#    0.1, 27/03/08 - Script initial                                  #
#                                                                    #
#--------------------------------------------------------------------#
# Scripts appelant    : n/a                                          #
# Scripts appeles     : n/a                                          #
# Fichiers en entree  : /tmp/usplash-theme-ubuntu_0.17.tar.gz        #
# Fichiers en sortie  : /tmp/usplash-theme-LiveCreator.so                    #
#--------------------------------------------------------------------#
######################################################################
# entre : $1 = image (.png / .bmp, autres)

image=$1

### Settings
THEME="LiveCreator"

### local functions

echo_msg()
{
	echo -n "[+] "
	echo $1
}
		
echo_error()
{
	echo -n "[E] " >&2
	echo $1 >&2
}

check_installed()
{
	echo -n "[+] Checking if $1 is already installed... "
	result=`dpkg-query -W $1 2>/dev/null | wc -l`
	if [ ${result} = "1" ]
	then
		echo "yes"
		return 1
	else
		echo "no"
		return 0
	fi
	return -1
}

if [ -z "${image}" ]
then
	echo_error "no file specified, using defaults"
	image="/tmp/splash.png"
fi

### dpkg -l imagemagick 

echo_msg "checking for packets..."
if check_installed "imagemagick" == 0
then
	imagemagick="imagemagick"
else
	imagemagick=""
fi

if check_installed "libc6-dev" == 0
then
     libc6dev="libc6-dev"
else
     libc6dev=""
fi

if check_installed "libusplash-dev" == 0
then
     libusplashdev="libusplash-dev"
else
     libusplashdev=""
fi

if check_installed "make" == 0
then
     libusplashdev="libusplash-dev"
else
     libusplashdev=""
fi

if check_installed "make" == 0
then
     make="make"
else
     make=""
fi

echo_msg "need to install: $imagemagick $libc6dev $libusplashdev $make"

echo_msg "preparing environment..."
#aptitude -y install usplash $imagemagick $libc6dev $libusplashdev $make
aptitude -y install usplash $libc6dev $libusplashdev $make
rm -rf /tmp/usplash-theme-ubuntu-0.16 /tmp/usplash-theme-${THEME}-1
rm -rf /tmp/usplash-theme-LiveCreator.so
tar -xzf /tmp/usplash-theme-ubuntu_0.17.tar.gz -C /tmp
mv /tmp/usplash-theme-ubuntu-0.16 /tmp/usplash-theme-${THEME}-1
#rm -f /tmp/usplash-theme-${THEME}-1/usplash_1024_768.png \
#	/tmp/usplash-theme-${THEME}-1/usplash_800_600.png \
#	/tmp/usplash-theme-${THEME}-1/usplash_640_480.png \
#	/tmp/usplash-theme-${THEME}-1/usplash_640x400.png \
#	/tmp/usplash-theme-${THEME}-1/usplash_1365_768_scaled.png \
#	/tmp/usplash-theme-${THEME}-1/usplash_1365_768.png \
#	/tmp/usplash-theme-${THEME}-1/usplash_primary.png

mv /tmp/usplash-theme-${THEME}-1/usplash-theme-ubuntu.c /tmp/usplash-theme-${THEME}-1/usplash-theme-${THEME}.c
sed -i "s/usplash-theme-ubuntu/usplash-theme-${THEME}/g" /tmp/usplash-theme-${THEME}-1/Makefile

echo_msg "creating images..."
#convert ${image} -depth 8 +map /tmp/usplash-theme-${THEME}-1/usplash_primary.png
#convert ${image} -depth 8 +map -resize 1024x768\! /tmp/usplash-theme-${THEME}-1/usplash_1024_768.png
#convert ${image} -depth 8 +map -resize 800x600\! /tmp/usplash-theme-${THEME}-1/usplash_800_600.png
#convert ${image} -depth 8 +map -resize 640x480\! /tmp/usplash-theme-${THEME}-1/usplash_640_480.png
#convert ${image} -depth 8 +map -resize 640x400\> -gravity center -crop 640x400+0+0 /tmp/usplash-theme-${THEME}-1/usplash_640x400.png
#convert ${image} -depth 8 +map -resize 1365x768\! /tmp/usplash-theme-${THEME}-1/usplash_1365_768_scaled.png
#convert ${image} -depth 8 +map -resize 1365x768\> -gravity center -crop 1365x768+0+0 /tmp/usplash-theme-${THEME}-1/usplash_1365_768.png
mv /tmp/splash.png /tmp/usplash-theme-${THEME}-1/usplash_640_480.png # HAX

echo_msg "building..."
cd /tmp/usplash-theme-${THEME}-1/ > /dev/null
make clean
make > /dev/null 2>&1
cd - > /dev/null

echo_msg "moving stuff..."
mkdir -p /usr/lib/usplash/
mv /tmp/usplash-theme-${THEME}-1/usplash-theme-LiveCreator.so /usr/lib/usplash/

echo_msg "cleaning up..."
rm -rf /tmp/usplash-theme-${THEME}-1
rm -f /tmp/usplash-theme-ubuntu_0.17.tar.gz
rm -f /tmp/splash.png
#aptitude -y purge $imagemagick $libc6dev $libusplashdev $make
aptitude -y purge $libc6dev $libusplashdev $make
