#!/bin/bash

######################################################################
# mountall_forensic.sh                                 Erwan Le Gall #
# 21/03/2008                                    elegall@linagora.com #
#                                                                    #
# Description:                                                       #
# Try to mount all standards devices and LVM devices with some       #
# Forensic specials options.                                         #
#                                                                    #
# IN :  -                                                            #
# OUT : -                                                            #
#                                                                    #
#--------------------------------------------------------------------#
# Copyright (C) elegall@linagora.com      <elegall@linagora.com>     #
# Copyright (C) LINAGORA                  <http://www.linagora.com>  #
#                                                                    #
#--------------------------------------------------------------------#
# License: GPLv3                                                     #
#--------------------------------------------------------------------#
# This program is free software; you can redistribute it and/or      #
# modify it under the terms of the GNU General Public License        #
# as published by the Free Software Foundation; either version 2     #
# of the License, or (at your option) any later version.             #
#                                                                    #
# This program is distributed in the hope that it will be useful,    #
# but WITHOUT ANY WARRANTY; without even the implied warranty of     #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      #
# GNU General Public License for more details.                       #
#                                                                    #
# GPL License: http://www.gnu.org/licenses/gpl.txt                   #
#                                                                    #
#--------------------------------------------------------------------#
# Please visit http://www.linagora.org for other contributions       #
#                                                                    #
#--------------------------------------------------------------------#
# Versions            :                                              #
#    1.0 - mountall_forensic.sh, 21/03/2008 - Script initial         #
#                                                                    #
#--------------------------------------------------------------------#
#                                                                    #
######################################################################


MOUNT_OPTIONS="noexec,noatime,nodev,nosuid"

# Classical disks
# using "mount" cause "/etc/mtab" don't exist on some unix.
DISKS="$(sudo fdisk -l | grep "^/dev/" | awk '{ print $1 }')"
MOUNTED="$(sudo mount | grep "^/dev/" | awk '{ print $1 }')"
#
for disk in ${DISKS}
do
    mounted=0
    for mounted_disk in ${MOUNTED}
    do
        if [ ${disk} = ${mounted_disk} ]
        then
            echo "${disk} déjà monté"
            mounted=1
        fi
    done
    if [ ${mounted} = 0 ]
    then
        echo "montage automatique de ${disk} dans /mnt/${disk##/dev/}"
        sudo mkdir -p /mnt/${disk##/dev/}
        sudo mount ${disk} /mnt/${disk##/dev/} -r -o ${MOUNT_OPTIONS}
    fi
done

# Now trying to guess LVM volumes
DISKS="$(ls /dev/mapper | grep -v "^control$")"
MOUNTED="$(sudo mount | grep "^/dev/" | awk '{ print $1 }')"

for disk in ${DISKS}
do
    mounted=0
    for mounted_disk in ${MOUNTED}
    do
        if [ "/dev/mapper/${disk}" = ${mounted_disk} ]
        then
            echo "/dev/mapper/${disk} déjà monté"
            mounted=1
        fi
    done
    if [ ${mounted} = 0 ]
    then
        echo "montage automatique de /dev/mapper/${disk} dans /mnt/${disk##/dev/}"
        sudo mkdir -p /mnt/${disk##/dev/}
        sudo mount /dev/mapper/${disk} /mnt/${disk##/dev/} -r -o ${MOUNT_OPTIONS}
    fi
done

exit 0
